﻿using System.ComponentModel;

namespace Databinding.ViewModel
{
    public class EmailViewModel : INotifyPropertyChanged
    {
        private string _subject;
        private string _body;

        private readonly DelegateCommand<string> _insertGreetingCommand;

        /// <summary>
        /// Initializes a new instance of the <see cref="EmailViewModel"/> class.
        /// </summary>
        public EmailViewModel()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="EmailViewModel"/> class.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        public EmailViewModel(string subject, string body)
        {
            _subject = subject;
            _body = body;

            _insertGreetingCommand =
                new DelegateCommand<string>(
                    s => { Body = string.Format("<strong>Hello my dear friend!</strong><br/>" + Body); }, s => true);
        }

        /// <summary>
        /// Gets or sets the subject.
        /// </summary>
        /// <value>
        /// The subject.
        /// </value>
        public string Subject
        {
            get { return _subject; }
            set
            {
                if (_subject == value)
                    return;

                _subject = value;
                OnPropertyChanged("Subject");
            }
        }

        /// <summary>
        /// Gets or sets the body.
        /// </summary>
        /// <value>
        /// The body.
        /// </value>
        public string Body
        {
            get { return _body; }
            set
            {
                if (_body == value)
                    return;

                _body = value;
                OnPropertyChanged("Body");
            }
        }

        /// <summary>
        /// Gets the insert greeting command.
        /// </summary>
        /// <value>
        /// The insert greeting command.
        /// </value>
        public DelegateCommand<string> InsertGreetingCommand
        {
            get { return _insertGreetingCommand; }
        }


        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Called when [property changed].
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
           
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("Subject: {0}\nBody: {1}", Subject, Body);
        }
    }
}