﻿using System.ComponentModel;

namespace Databinding.ViewModel
{
    public class OneWayViewModel : INotifyPropertyChanged
    {
        private string _bodyHtml;

        /// <summary>
        /// Gets or sets the body HTML.
        /// </summary>
        /// <value>
        /// The body HTML.
        /// </value>
        public string BodyHtml
        {
            get
            {
                return _bodyHtml;
            }
            set
            {
                if (_bodyHtml == value) 
                    return;
               
                _bodyHtml = value;
                OnPropertyChanged("BodyHtml");
            }
        }


        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Called when [property changed].
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            
            if (handler != null) 
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
