﻿Imports System.Windows.Input

Public Class DelegateCommand(Of T As Class)
    Implements ICommand
    Private ReadOnly _canExecute As Predicate(Of T)
    Private ReadOnly _execute As Action(Of T)

    ''' <summary>
    ''' Initializes a new instance of the <see cref="DelegateCommand{T}"/> class.
    ''' </summary>
    ''' <param name="execute">The execute.</param>
    ''' <param name="canExecute">The can execute.</param>
    Public Sub New(execute As Action(Of T), Optional canExecute As Predicate(Of T) = Nothing)
        _execute = execute
        _canExecute = canExecute
    End Sub

    ''' <summary>
    ''' Defines the method that determines whether the command can execute in its current state.
    ''' </summary>
    ''' <param name="parameter">Data used by the command.  If the command does not require data to be passed, this object can be set to null.</param>
    ''' <returns>
    ''' true if this command can be executed; otherwise, false.
    ''' </returns>
    Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
        Return _canExecute Is Nothing OrElse _canExecute(DirectCast(parameter, T))
    End Function

    Public Event ICommand_CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    ''' <summary>
    ''' Defines the method to be called when the command is invoked.
    ''' </summary>
    ''' <param name="parameter">Data used by the command.  If the command does not require data to be passed, this object can be set to null.</param>
    Public Sub Execute(parameter As Object) Implements ICommand.Execute
        _execute(DirectCast(parameter, T))
    End Sub


    Public Event CanExecuteChanged As EventHandler
End Class