﻿
Imports System.ComponentModel

Namespace ViewModel
    Public Class EmailViewModel
        Implements INotifyPropertyChanged
        Private _subject As String
        Private _body As String

        Private ReadOnly _insertGreetingCommand As DelegateCommand(Of String)

        ''' <summary>
        ''' Initializes a new instance of the <see cref="EmailViewModel"/> class.
        ''' </summary>
        Public Sub New()
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="EmailViewModel"/> class.
        ''' </summary>
        ''' <param name="subject">The subject.</param>
        ''' <param name="body__1">The body.</param>
        Public Sub New(subject As String, body__1 As String)
            _subject = subject
            _body = body__1

            _insertGreetingCommand = New DelegateCommand(Of String)(Function(s)
                                                                        Body = String.Format(Convert.ToString("<strong>Hello my dear friend!</strong><br/>") & Body)

                                                                    End Function, Function(s) True)
        End Sub

        ''' <summary>
        ''' Gets or sets the subject.
        ''' </summary>
        ''' <value>
        ''' The subject.
        ''' </value>
        Public Property Subject() As String
            Get
                Return _subject
            End Get
            Set(value As String)
                If _subject = value Then
                    Return
                End If

                _subject = value
                OnPropertyChanged("Subject")
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the body.
        ''' </summary>
        ''' <value>
        ''' The body.
        ''' </value>
        Public Property Body() As String
            Get
                Return _body
            End Get
            Set(value As String)
                If _body = value Then
                    Return
                End If

                _body = value
                OnPropertyChanged("Body")
            End Set
        End Property

        ''' <summary>
        ''' Gets the insert greeting command.
        ''' </summary>
        ''' <value>
        ''' The insert greeting command.
        ''' </value>
        Public ReadOnly Property InsertGreetingCommand() As DelegateCommand(Of String)
            Get
                Return _insertGreetingCommand
            End Get
        End Property


        ''' <summary>
        ''' Occurs when a property value changes.
        ''' </summary>
        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        ''' <summary>
        ''' Called when [property changed].
        ''' </summary>
        ''' <param name="propertyName">Name of the property.</param>
        Protected Overridable Sub OnPropertyChanged(propertyName As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
        End Sub

        ''' <summary>
        ''' Returns a <see cref="System.String" /> that represents this instance.
        ''' </summary>
        ''' <returns>
        ''' A <see cref="System.String" /> that represents this instance.
        ''' </returns>
        Public Overrides Function ToString() As String
            Return String.Format("Subject: {0}" & vbLf & "Body: {1}", Subject, Body)
        End Function


    End Class
End Namespace