﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using SpiceLogic.WpfHtmlEditor.Domain.Dialogs;

namespace Dialog_Customization.Dialogs.ColorPickerDialog
{
    /// <summary>
    /// Interaction logic for ColorPickerDialog.xaml
    /// </summary>
    public partial class ColorPickerDialog : IColorPickerDialog
    {
        private Color _mColor;
        private readonly Color _startingColor = new Color();

        public ColorPickerDialog()
        {
            InitializeComponent();
        }

        
        private void onSelectedColorChanged(object sender, RoutedPropertyChangedEventArgs<Color> e)
        {
            if (e.NewValue != _mColor)
                TwoButtonPanel.LeftButton.IsEnabled = true;
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            TwoButtonPanel.LeftButton.IsEnabled = false;
            base.OnClosing(e);
        }

        /// <summary>
        /// The selected color
        /// </summary>
        public Color SelectedColor
        {
            get
            {
                return _mColor;
            }
        }

        /// <summary>
        /// The initial color that will be selected when the dialog opens
        /// </summary>
        public Color StartingColor
        {
            get
            {
                return _startingColor;
            }
            set
            {
                cPicker.SelectedColor = value;

                if (TwoButtonPanel.LeftButton != null)
                    TwoButtonPanel.LeftButton.IsEnabled = false;
            }
        }

        private void TwoButtonPanel_OnOnRightButtonClicked(object sender, RoutedEventArgs e)
        {
            //Cancel button

            TwoButtonPanel.LeftButton.IsEnabled = false;
            DialogResult = false;
        }

        private void TwoButtonPanel_OnOnLeftButtonClicked(object sender, RoutedEventArgs e)
        {
			 //OK button

            TwoButtonPanel.LeftButton.IsEnabled = false;
            _mColor = cPicker.SelectedColor;
            DialogResult = true;
            Hide();
        }

        private void DialogHeader_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            DragMove();
        }

        public void Dispose()
        {
        }
    }
}