﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using SpiceLogic.HtmlEditorControl.EditorShared.Dialogs.StyleBuilder;
using SpiceLogic.WpfHtmlEditor.Extensions;
using SpiceLogic.WpfHtmlEditor.Domain.Dialogs;
using SpiceLogic.WpfHtmlEditor.ToolbarModule;

namespace Dialog_Customization.Dialogs.StyleBuilder
{
    /// <summary>
    /// Class ucFont
    /// </summary>
    [ToolboxItem(false)]
    [FormSelectorPage("Font", "font;font-family;font-size;text-decoration;font-weight;text-transform;color;font-style;font-variant")]
    public partial class ucFont : IEditorStylePage
    {
        /// <summary>
        /// The _dict
        /// </summary>
        readonly Dictionary<string, string> _dict;

        /// <summary>
        /// A method creating the color dialog
        /// </summary>
        private readonly DialogFactoryDelegates.CreateColorPickerDialogDelegate _createColorDialogMethod;

        #region Preset of possible value

        /// <summary>
        /// The _ system fonts
        /// </summary>
        readonly List<KeyValuePair<string, string>> _SystemFonts = new List<KeyValuePair<string, string>>();
        /// <summary>
        /// The _ font style
        /// </summary>
        readonly List<KeyValuePair<string, string>> _FontStyle = new List<KeyValuePair<string, string>>();
        /// <summary>
        /// The _ font variant
        /// </summary>
        readonly List<KeyValuePair<string, string>> _FontVariant = new List<KeyValuePair<string, string>>();
        /// <summary>
        /// The _ bold absolute
        /// </summary>
        readonly List<KeyValuePair<string, string>> _BoldAbsolute = new List<KeyValuePair<string, string>>();
        /// <summary>
        /// The _ bold relative
        /// </summary>
        readonly List<KeyValuePair<string, string>> _BoldRelative = new List<KeyValuePair<string, string>>();
        /// <summary>
        /// The _ text transform
        /// </summary>
        readonly List<KeyValuePair<string, string>> _TextTransform = new List<KeyValuePair<string, string>>();
        #endregion

        /// <summary>
        /// Creates the lists.
        /// </summary>
        protected void CreateLists()
        {
            #region Initialize presets
            _SystemFonts.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _SystemFonts.Add(new KeyValuePair<string, string>("Window caption", "caption"));
            _SystemFonts.Add(new KeyValuePair<string, string>("ToolWindow caption", "small-caption"));
            _SystemFonts.Add(new KeyValuePair<string, string>("Dialog text", "message-box"));
            _SystemFonts.Add(new KeyValuePair<string, string>("Icon labels", "icon"));
            _SystemFonts.Add(new KeyValuePair<string, string>("Menu text", "menu"));
            _SystemFonts.Add(new KeyValuePair<string, string>("Tooltip text", "status-bar"));

            _FontStyle.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _FontStyle.Add(new KeyValuePair<string, string>("Normal", "normal"));
            _FontStyle.Add(new KeyValuePair<string, string>("Italic", "italic"));

            _FontVariant.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _FontVariant.Add(new KeyValuePair<string, string>("Normal", "normal"));
            _FontVariant.Add(new KeyValuePair<string, string>("Small Caps", "small-caps"));

            _BoldAbsolute.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _BoldAbsolute.Add(new KeyValuePair<string, string>("Normal", "normal"));
            _BoldAbsolute.Add(new KeyValuePair<string, string>("Bold", "bold"));

            _BoldRelative.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _BoldRelative.Add(new KeyValuePair<string, string>("Lighter", "lighter"));
            _BoldRelative.Add(new KeyValuePair<string, string>("Bolder", "bolder"));

            _TextTransform.Add(new KeyValuePair<string, string>("<Not Set>", ""));
            _TextTransform.Add(new KeyValuePair<string, string>("None", "none"));
            _TextTransform.Add(new KeyValuePair<string, string>("Initial Cap", "capitalize"));
            _TextTransform.Add(new KeyValuePair<string, string>("lowercase", "lowercase"));
            _TextTransform.Add(new KeyValuePair<string, string>("UPPERCASE", "uppercase"));
            #endregion
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ucFont"/> class.
        /// </summary>
        /// <param name="dict">The dict.</param>
        /// <param name="createColorDialogMethod">A method creating the color dialog</param>
        public ucFont(Dictionary<string, string> dict, DialogFactoryDelegates.CreateColorPickerDialogDelegate createColorDialogMethod) //the argument order is important (Activator.CreateInstance uses it)
        {
            _dict = dict;
            _createColorDialogMethod = createColorDialogMethod;
            CreateLists();
            InitializeComponent();

        }

        /// <summary>
        /// Flushes the content of the user control back to the dictionary
        /// </summary>
        public void FlushContent()
        {
            _dict.Remove("font-family");
            _dict.Remove("font");
            _dict.Remove("color");
            _dict.Remove("font-style");
            _dict.Remove("font-variant");
            _dict.Remove("font-weight");
            _dict.Remove("font-size");
            _dict.Remove("text-decoration");

            {   // handle text decoration
                StringBuilder sb = new StringBuilder();
                if (CbEffectNone.IsChecked == true)
                    sb.Append(" none");
                if (CbEffectUnderline.IsChecked == true)
                    sb.Append(" underline");
                if (CbEffectStrikethrough.IsChecked == true)
                    sb.Append(" line-through");
                if (CbEffectOverline.IsChecked == true)
                    sb.Append(" overline");
                _dict["text-decoration"] = sb.ToString();
            }

            _dict["text-transform"] = (string)CbCapitalization.SelectedValue;

            if (RbFamily.IsChecked == true)
            {
                _dict["font-family"] = TbFontFamily.Text;
                _dict["color"] = WpfColorTranslator.ToHtml(((SolidColorBrush)TxtForeColor.Background).Color); // cbColor.Text.ToLowerInvariant();
                _dict["font-style"] = (string)CbFontStyle.SelectedValue;
                _dict["font-variant"] = (string)CbFontVariant.SelectedValue;

                // font-size
                if (RbSizeSpecific.IsChecked == true)
                {
                    if (TbSpecificSize.Text.Trim().Length > 0)
                        _dict["font-size"] = TbSpecificSize.Text + CbSpecificSizeType.Text;
                }
                else if (RbSizeAbsolute.IsChecked == true)
                    _dict["font-size"] = CbAbsoluteSize.Text.ToLowerInvariant();
                else
                    _dict["font-size"] = CbRelativeSize.Text.ToLowerInvariant();

                // font-weight
                if (RbBoldAbsolute.IsChecked == true)
                    _dict["font-weight"] = (string)CbBoldAbsolute.SelectedValue;
                else
                    _dict["font-weight"] = (string)CbBoldRelative.SelectedValue;
            }

            if (RbSystemFont.IsChecked == true)
                _dict["font"] = (string)CbSystemFont.SelectedValue;
        }

        /// <summary>
        /// Handles the Loaded event of the ucFont control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void ucFont_Loaded(object sender, RoutedEventArgs e)
        {
            #region set data sources
            CbSystemFont.ItemsSource = _SystemFonts;
            CbSystemFont.DisplayMemberPath = "Key";
            CbSystemFont.SelectedValuePath = "Value";
            CbSystemFont.SelectedIndex = 0;

            CbFontStyle.ItemsSource = _FontStyle;
            CbFontStyle.DisplayMemberPath = "Key";
            CbFontStyle.SelectedValuePath = "Value";
            CbFontStyle.SelectedIndex = 0;

            CbFontVariant.ItemsSource = _FontVariant;
            CbFontVariant.DisplayMemberPath = "Key";
            CbFontVariant.SelectedValuePath = "Value";
            CbFontVariant.SelectedIndex = 0;

            CbBoldAbsolute.ItemsSource = _BoldAbsolute;
            CbBoldAbsolute.DisplayMemberPath = "Key";
            CbBoldAbsolute.SelectedValuePath = "Value";
            CbBoldAbsolute.SelectedIndex = 0;

            CbBoldRelative.ItemsSource = _BoldRelative;
            CbBoldRelative.DisplayMemberPath = "Key";
            CbBoldRelative.SelectedValuePath = "Value";
            CbBoldRelative.SelectedIndex = 0;

            CbCapitalization.ItemsSource = _TextTransform;
            CbCapitalization.DisplayMemberPath = "Key";
            CbCapitalization.SelectedValuePath = "Value";
            CbCapitalization.SelectedIndex = 0;
            #endregion

            #region set default values
            RbFamily.IsChecked = true;
            RbSizeSpecific.IsChecked = true;
            RbBoldAbsolute.IsChecked = true;
            #endregion

            string value;

            #region parse

            if (_dict.TryGetValue("color", out value))
                TxtForeColor.Background = new SolidColorBrush(WpfColorTranslator.FromHtml(value));

            if (_dict.TryGetValue("font-style", out value))
            {
                for (int i = 0, n = _FontStyle.Count; i < n; ++i)
                {
                    if (string.Equals(value, _FontStyle[i].Value, StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbFontStyle.SelectedIndex = i;
                        break;
                    }
                }
            }

            if (_dict.TryGetValue("font-size", out value))
            {
                bool handled = false;

                for (int i = 0, n = CbAbsoluteSize.Items.Count; i < n && !handled; ++i)
                    if (string.Equals(value, (CbAbsoluteSize.Items[i] as ComboBoxItem).Content as string, StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbAbsoluteSize.SelectedIndex = i;
                        RbSizeAbsolute.IsChecked = true;
                        handled = true;
                    }

                for (int i = 0, n = CbRelativeSize.Items.Count; i < n && !handled; ++i)
                    if (string.Equals(value, (CbRelativeSize.Items[i] as ComboBoxItem).Content as string, StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbRelativeSize.SelectedIndex = i;
                        RbSizeRelative.IsChecked = true;
                        handled = true;
                    }

                if (!handled)
                {
                    RbSizeSpecific.IsChecked = true;
                    for (int i = 0, n = CbSpecificSizeType.Items.Count; i < n && !handled; ++i)
                    {
                        string cbSpecificSizeTypeItem = (CbSpecificSizeType.Items[i] as ComboBoxItem).Content as string;
                        if (!string.IsNullOrEmpty(cbSpecificSizeTypeItem) && value.EndsWith(cbSpecificSizeTypeItem, StringComparison.InvariantCultureIgnoreCase))
                        {
                            CbSpecificSizeType.SelectedIndex = i;
                            TbSpecificSize.Text = value.Substring(0, value.Length - cbSpecificSizeTypeItem.Length);
                            handled = true;
                        }
                    }
                }
            }

            if (_dict.TryGetValue("font-weight", out value))
            {
                bool handled = false;

                for (int i = 0, n = _BoldAbsolute.Count; i < n && !handled; ++i)
                {
                    if (string.Equals(value, _BoldAbsolute[i].Value, StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbBoldAbsolute.SelectedIndex = i;
                        RbBoldAbsolute.IsChecked = true;
                        handled = true;
                    }
                }

                for (int i = 0, n = _BoldRelative.Count; i < n && !handled; ++i)
                {
                    if (string.Equals(value, _BoldRelative[i].Value, StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbBoldRelative.SelectedIndex = i;
                        RbBoldRelative.IsChecked = true;
                        handled = true;
                    }
                }
            }

            if (_dict.TryGetValue("text-decoration", out value))
            {
                string loValue = value.ToLowerInvariant();
                CbEffectUnderline.IsChecked = loValue.Contains("underline");
                CbEffectStrikethrough.IsChecked = loValue.Contains("line-through") || loValue.Contains("linethrough");
                CbEffectOverline.IsChecked = loValue.Contains("overline");
                CbEffectNone.IsChecked = loValue.Contains("none");
            }

            if (_dict.TryGetValue("text-transform", out value))
            {
                for (int i = 0, n = _TextTransform.Count; i < n; ++i)
                    if (string.Equals(value, _TextTransform[i].Value, StringComparison.InvariantCultureIgnoreCase))
                        CbCapitalization.SelectedIndex = i;
            }

            if (_dict.TryGetValue("font-family", out value))
            {
                RbFamily.IsChecked = true;
                TbFontFamily.Text = value;
            }

            if (_dict.TryGetValue("font", out value))
            {
                RbSystemFont.IsChecked = true;

                for (int i = 0, n = _SystemFonts.Count; i < n; ++i)
                {
                    if (string.Equals((value ?? "").Replace("-", "").Trim(), _SystemFonts[i].Value.Replace("-", ""), StringComparison.InvariantCultureIgnoreCase))
                    {
                        CbSystemFont.SelectedIndex = i;
                        break;
                    }
                }
            }
            #endregion
        }

        #region UI handlers
        /// <summary>
        /// Handles the Click event of the BtFontFamilySelect control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void btFontFamilySelect_Click(object sender, RoutedEventArgs e)
        {
            FontPicker subForm = new FontPicker(TbFontFamily.Text);
            if (subForm.ShowDialog() == true)
                TbFontFamily.Text = subForm.SelectedFontList;
        }

        /// <summary>
        /// Fonts the type changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void FontTypeChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            TbFontFamily.IsEnabled = btFontFamilySelect.IsEnabled = RbFamily.IsChecked == true;
            CbSystemFont.IsEnabled = !RbFamily.IsChecked == true;

            GbSize.IsEnabled = GbBold.IsEnabled = CbFontStyle.IsEnabled = CbFontVariant.IsEnabled = RbFamily.IsChecked == true;
        }

        /// <summary>
        /// Handles the Checked and Unchecked events of the RbSizeSpecific control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void rbSizeSpecific_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            TbSpecificSize.IsEnabled = CbSpecificSizeType.IsEnabled = true;
            CbRelativeSize.IsEnabled = CbAbsoluteSize.IsEnabled = false;
        }

        /// <summary>
        /// Handles the Checked and Unchecked events of the RbSizeAbsolute control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void rbSizeAbsolute_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            CbAbsoluteSize.IsEnabled = true;
            TbSpecificSize.IsEnabled = CbSpecificSizeType.IsEnabled = CbRelativeSize.IsEnabled = false;
        }

        /// <summary>
        /// Handles the Checked and Unchecked events of the RbSizeRelative control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void rbSizeRelative_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            CbRelativeSize.IsEnabled = true;
            TbSpecificSize.IsEnabled = CbSpecificSizeType.IsEnabled = CbAbsoluteSize.IsEnabled = false;
        }

        /// <summary>
        /// Handles the Checked and Unchecked events of the cbEffectNone control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void cbEffectNone_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            if (CbEffectNone.IsChecked == true)
            {
                CbEffectOverline.IsChecked = CbEffectStrikethrough.IsChecked = CbEffectUnderline.IsChecked = false;
                CbEffectOverline.IsEnabled = CbEffectStrikethrough.IsEnabled = CbEffectUnderline.IsEnabled = false;
            }
            else
            {
                CbEffectOverline.IsEnabled = CbEffectStrikethrough.IsEnabled = CbEffectUnderline.IsEnabled = true;
            }
        }

        /// <summary>
        /// Rbs the bold radio button changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void rbBoldRadioButtonChanged(object sender, RoutedEventArgs e)
        {
            if (!IsInitialized)
                return;
            
            CbBoldAbsolute.IsEnabled = RbBoldAbsolute.IsChecked == true;
            CbBoldRelative.IsEnabled = RbBoldRelative.IsChecked == true;
        }
        #endregion

        
        /// <summary>
        /// Handles the PreviewMouseLeftButtonUp events of the TxtForeColor control.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void TxtForeColor_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            SolidColorBrush solidColorBrush = TxtForeColor.Background as SolidColorBrush;
            if (solidColorBrush != null)
            {
                using (IColorPickerDialog colorDialog = _createColorDialogMethod())
                {
                    colorDialog.StartingColor = solidColorBrush.Color;
                    if (colorDialog.ShowDialog() == true)
                        TxtForeColor.Background = new SolidColorBrush(colorDialog.SelectedColor);
                }
            }
        }
    }

    
}
