﻿Imports System.Globalization
Imports System.IO
Imports System.Text.RegularExpressions
Imports Microsoft.Win32
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Services
Imports Color = System.Drawing.Color
Imports Size = System.Drawing.Size

Namespace Dialogs
    ''' <summary>
    ''' Class ImageDialog
    ''' </summary>
    Partial Public Class ImageDialog
        Implements IImageDialog
        ''' <summary>
        ''' The _the original element
        ''' </summary>
        Private _theOriginalElement As ImageElement
        ''' <summary>
        ''' The _width to height aspect ratio
        ''' </summary>
        Private _widthToHeightAspectRatio As Nullable(Of Single)

        ''' <summary>
        ''' The dialog service
        ''' </summary>
        Private ReadOnly _dialogService As IDialogService

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ImageDialog" /> class.
        ''' </summary>
        ''' <param name="dialogService">The dialog service.</param>
        Public Sub New(dialogService As IDialogService)
            _dialogService = dialogService

            InitializeComponent()
        End Sub


        ''' <summary>
        ''' Handles the MouseLeftButtonDown event of the DialogHeader control
        ''' </summary>
        ''' <param name="sender">The evetn source</param>
        ''' <param name="e">The event data</param>
        Private Sub DialogHeader_MouseLeftButtonDown(sender As Object, e As MouseButtonEventArgs)
            DragMove()
        End Sub

        ''' <summary>
        ''' Handles the Loaded event of the ImageInsertDialog control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub ImageInsertDialog_Loaded(sender As Object, e As RoutedEventArgs)
            If String.IsNullOrEmpty(_theOriginalElement.BaseURL) Then
                RdoWorkingDirFile.IsEnabled = False
                RdoWorkingDirFile.ToolTip = "You need to set Base Url in order to use this option"
                RdoWorkingDirFile.SetValue(ToolTipService.ShowOnDisabledProperty, True)
            End If
        End Sub

        ''' <summary>
        ''' Handles the Click event of the BtnBrowseFile control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub btnBrowseFile_Click(sender As Object, e As RoutedEventArgs)
            Dim srcUrlDialog As New OpenFileDialog() With {
                .RestoreDirectory = True,
                .Filter = "Image Files|*.png;*.bmp;*.gif;*.jpg|All files(*.*)|*.*",
                .FilterIndex = 0,
                .Multiselect = False
            }

            If srcUrlDialog.ShowDialog() = True Then
                TxtUrl.Text = srcUrlDialog.FileName
                Dim imageFileName As String = srcUrlDialog.FileName
                setImageDimensionaAndAspectRatio(imageFileName)
            End If
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the RdoLocalFile control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub rdoLocalFile_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            updateBrowserButtonState()
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the RdoWorkingDirFile control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub rdoWorkingDirFile_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            Dim isLocalDir As Nullable(Of Boolean) = Me._theOriginalElement.IsBaseUrlALocalFolder()

            LnkBtnImportToBaseFolder.IsEnabled = RdoWorkingDirFile.IsEnabled AndAlso RdoWorkingDirFile.IsChecked = True AndAlso isLocalDir.HasValue AndAlso isLocalDir.Value
            ChkOverwrite.IsEnabled = RdoWorkingDirFile.IsEnabled AndAlso RdoWorkingDirFile.IsChecked = True AndAlso isLocalDir.HasValue AndAlso isLocalDir.Value
            LnkBtnBrowseWD.IsEnabled = RdoWorkingDirFile.IsEnabled AndAlso RdoWorkingDirFile.IsChecked = True AndAlso isLocalDir.HasValue AndAlso isLocalDir.Value
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkBtnBrowseWD control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="routedEventArgs">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        ''' <exception cref="Exception">File doesn't exist</exception>
        Private Sub lnkBtnBrowseWD_Click(sender As Object, routedEventArgs As RoutedEventArgs)
            Dim srcUrlDialog As New OpenFileDialog() With {
                .RestoreDirectory = True,
                .Filter = "Image Files|*.png;*.bmp;*.gif;*.jpg|All files(*.*)|*.*",
                .FilterIndex = 0,
                .Multiselect = False,
                .InitialDirectory = Me._theOriginalElement.BaseURL
            }

            If srcUrlDialog.ShowDialog() = True Then
                Dim baseUrl As String = If(_theOriginalElement.BaseURL, String.Empty)
                If baseUrl.EndsWith("\") OrElse baseUrl.EndsWith("/") Then
                    baseUrl = baseUrl.Remove(baseUrl.Length - 1)
                End If

                Dim selectedFileDirectory = Path.GetDirectoryName(srcUrlDialog.FileName)
                If selectedFileDirectory Is Nothing Then
                    Return
                End If
                If (selectedFileDirectory.StartsWith(baseUrl, StringComparison.OrdinalIgnoreCase)) Then
                    Dim relativePath As String = srcUrlDialog.FileName.Replace(baseUrl, String.Empty)

                    If (relativePath.StartsWith("\\")) Then
                        relativePath = relativePath.Remove(0, 1)
                    End If

                    TxtUrl.Text = relativePath
                Else
                    Dim dlgResult As MessageBoxResult = MessageBox.Show("The image you selected is not from the base directory for relative path. Do you want to import that file to your base directory ? If you choose YES, then it will be imported to the Base Directory, otherwise it will be treated as absolute path image file.", "Selected image is not from the base directory.", MessageBoxButton.YesNoCancel, MessageBoxImage.Question)
                    If dlgResult = MessageBoxResult.Yes Then
                        Dim newFilePath As String = Path.Combine(baseUrl, If(Path.GetFileName(srcUrlDialog.FileName), String.Empty))
                        Dim i = 0
                        While File.Exists(newFilePath) AndAlso (ChkOverwrite.IsChecked.HasValue = False Or ChkOverwrite.IsChecked = False)
                            i += 1
                            Dim newFileName As String = Path.GetFileNameWithoutExtension(srcUrlDialog.FileName) + i + Path.GetExtension(srcUrlDialog.FileName)
                            newFilePath = Path.Combine(baseUrl, newFileName)
                        End While
                        If File.Exists(newFilePath) Then
                            Try
                                File.SetAttributes(newFilePath, FileAttributes.Normal)
                                File.Delete(newFilePath)
                                File.Copy(srcUrlDialog.FileName, newFilePath)
                            Catch err As Exception
                                MessageBox.Show(err.Message, "Error copying file to the destination")
                            End Try
                        Else
                            File.Copy(srcUrlDialog.FileName, newFilePath)
                        End If
                        TxtUrl.Text = Path.GetFileName(newFilePath)
                    ElseIf dlgResult = MessageBoxResult.No Then
                        TxtUrl.Text = srcUrlDialog.FileName
                    End If
                End If

                If Not String.IsNullOrEmpty(TxtUrl.Text) Then
                    Dim fullImagePath As String = If(File.Exists(TxtUrl.Text), TxtUrl.Text, Path.Combine(baseUrl, TxtUrl.Text))
                    If File.Exists(fullImagePath) Then
                        setImageDimensionaAndAspectRatio(fullImagePath)
                    Else
                        Throw New Exception("File doesn't exist")
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkAlignment control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkAlignment_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CmbAlign.IsEnabled = ChkAlignment.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderThickness control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderThickness_CheckedChanged(sender As Object, e As RoutedEventArgs)
            TxtBorder.IsEnabled = ChkBorderThickness.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkHeight control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkHeight_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtHeight.IsEnabled = ChkHeight.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkWidth_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtWidth.IsEnabled = ChkWidth.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderColor_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            LnkBtnBgColor.IsEnabled = ChkBorderColor.IsChecked = True
            TxtBgColor.IsEnabled = ChkBorderColor.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderStyle_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CmbBorderStyle.IsEnabled = ChkBorderStyle.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the TextChanged event of the TxtHeight control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub txtHeight_TextChanged(sender As Object, e As RoutedEventArgs)
            If TxtHeight.IsFocused AndAlso ChkLockAspectRatio.IsChecked = True AndAlso Me._widthToHeightAspectRatio.HasValue AndAlso Me._widthToHeightAspectRatio.Value > 0 Then
                Try
                    Dim value As String = TxtHeight.Text
                    Dim digitPart As String
                    Dim unitPart As String
                    getValueAndUnit(value, digitPart, unitPart)

                    If digitPart.Length > 0 Then
                        Dim height As Single = Single.Parse(digitPart)
                        If height > 0 Then
                            Dim width As Single = Me._widthToHeightAspectRatio.Value * height
                            TxtWidth.Text = CInt(Math.Round(width)) & unitPart
                        End If
                    End If
                Catch
                End Try
            End If
        End Sub

        ''' <summary>
        ''' Handles the TextChanged event of the TxtWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        Private Sub txtWidth_TextChanged(sender As Object, e As EventArgs)
            If TxtWidth.IsFocused AndAlso ChkLockAspectRatio.IsChecked = True AndAlso Me._widthToHeightAspectRatio.HasValue AndAlso Me._widthToHeightAspectRatio.Value > 0 Then
                Try
                    Dim value As String = TxtWidth.Text
                    Dim digitPart As String
                    Dim unitPart As String
                    getValueAndUnit(value, digitPart, unitPart)

                    If digitPart.Length > 0 Then
                        Dim width As Single = Single.Parse(digitPart)
                        If width > 0 Then
                            Dim height As Single = width / Me._widthToHeightAspectRatio.Value
                            TxtHeight.Text = CInt(Math.Round(height)) & unitPart
                        End If
                    End If
                Catch
                End Try
            End If
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkBtnImportToBaseFolder control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="routedEventArgs">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub lnkBtnImportToBaseFolder_Click(sender As Object, routedEventArgs As RoutedEventArgs)
            Dim myDialog As New OpenFileDialog() With {
                .RestoreDirectory = True,
                .Filter = "Image Files|*.png;*.bmp;*.gif;*.jpg|All files(*.*)|*.*",
                .FilterIndex = 0,
                .Multiselect = False
            }

            If myDialog.ShowDialog() = True Then
                Dim newFilePath As String = Path.Combine(Me._theOriginalElement.BaseURL, If(Path.GetFileName(myDialog.FileName), String.Empty))
                Dim i = 0
                While File.Exists(newFilePath) AndAlso (ChkOverwrite.IsChecked.HasValue = False Or ChkOverwrite.IsChecked = False)
                    i += 1
                    Dim newFileName As String = Path.GetFileNameWithoutExtension(myDialog.FileName) + i + Path.GetExtension(myDialog.FileName)
                    newFilePath = Path.Combine(Me._theOriginalElement.BaseURL, newFileName)
                End While
                If File.Exists(newFilePath) Then
                    Try
                        File.SetAttributes(newFilePath, FileAttributes.Normal)
                        File.Delete(newFilePath)
                        File.Copy(myDialog.FileName, newFilePath)
                    Catch err As Exception
                        MessageBox.Show(err.Message, "Error copying file to the destination")
                    End Try
                Else
                    File.Copy(myDialog.FileName, newFilePath)
                End If
                TxtUrl.Text = Path.GetFileName(newFilePath)
            End If
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As ImageElement Implements IImageDialog.Element
            Get
                Return readUi()
            End Get
            Set(value As ImageElement)
                Me._theOriginalElement = value
                Me.updateUi(value)
            End Set
        End Property

        ''' <summary>
        ''' Reads the UI.
        ''' </summary>
        ''' <returns>ImageElement.</returns>
        Private Function readUi() As ImageElement
            Dim src As String = Nothing

            If ChkInsertLocalBase64.IsChecked = True Then
                Try
                    If File.Exists(TxtUrl.Text) Then
                        src = ImageElement.GetBase64DataUrlForLocalImage(TxtUrl.Text)
                    End If
                    ' ignored
                Catch generatedExceptionName As Exception
                End Try
            End If

            Dim theElement As New ImageElement() With {
                .TheActiveHTMLElement = Me._theOriginalElement.TheActiveHTMLElement,
                .CSSStyle = Me._theOriginalElement.CSSStyle,
                .CSSClassName = Me._theOriginalElement.CSSClassName,
                .Name = Me._theOriginalElement.Name,
                .ID = Me._theOriginalElement.ID,
                .OnClickJavascript = Me._theOriginalElement.OnClickJavascript,
                .SrcUrl = If(src, TxtUrl.Text)
            }

            If ChkWidth.IsChecked = True Then
                theElement.Width = TxtWidth.Text.Trim()
            End If
            If ChkHeight.IsChecked = True Then
                theElement.Height = TxtHeight.Text.Trim()
            End If
            If ChkBorderColor.IsChecked = True Then
                Dim bgBrush = TryCast(TxtBgColor.Background, SolidColorBrush)
                If bgBrush IsNot Nothing Then
                    theElement.BorderColor = Color.FromArgb(bgBrush.Color.A, bgBrush.Color.R, bgBrush.Color.G, bgBrush.Color.B)
                End If
            End If

            If ChkBorderStyle.IsChecked = True Then
                theElement.BorderStyle = CmbBorderStyle.Text
            End If
            If ChkBorderThickness.IsChecked = True AndAlso (Not String.IsNullOrEmpty(TxtBorder.Text)) Then
                theElement.Border = Convert.ToInt32(TxtBorder.Text)
            End If
            theElement.Title = TxtToolTip.Text.Trim()
            theElement.AlternativeText = TxtAlt.Text.Trim()
            If ChkAlignment.IsChecked = True AndAlso (Not String.IsNullOrEmpty(CmbAlign.Text)) Then
                theElement.Align = CmbAlign.Text
            End If
            Return theElement
        End Function

        ''' <summary>
        ''' Updates the UI.
        ''' </summary>
        ''' <param name="element">The element.</param>
        Private Sub updateUi(element As ImageElement)
            If Me.IsLocalResourceSelectionDisabled Then
                RdoLocalFile.IsEnabled = False
                BtnBrowseFile.IsEnabled = False
            End If

            TxtUrl.Text = element.SrcUrl
            If element.IsRelativePathOrUrl Then
                RdoWorkingDirFile.IsChecked = True
            ElseIf element.IsLocalFilePath AndAlso Not Me.IsLocalResourceSelectionDisabled Then
                RdoLocalFile.IsChecked = True
            End If
            TxtToolTip.Text = element.Title
            TxtAlt.Text = element.AlternativeText
            CmbAlign.Text = element.Align
            ChkAlignment.IsChecked = (Not String.IsNullOrEmpty(element.Align))
            TxtBorder.Text = If(element.Border.HasValue, element.Border.Value.ToString(CultureInfo.InvariantCulture), String.Empty)
            ChkBorderThickness.IsChecked = (Not String.IsNullOrEmpty(TxtBorder.Text))
            TxtWidth.Text = element.Width
            ChkWidth.IsChecked = (Not String.IsNullOrEmpty(element.Width))
            TxtHeight.Text = element.Height
            ChkHeight.IsChecked = (Not String.IsNullOrEmpty(element.Height))
            ChkBorderColor.IsChecked = element.BorderColor.HasValue

            If element.BorderColor.HasValue Then
                Dim bgColor As Media.Color = Media.Color.FromArgb(element.BorderColor.Value.A, element.BorderColor.Value.R, element.BorderColor.Value.G, element.BorderColor.Value.B)
                Dim bgBrush As New SolidColorBrush(bgColor)
                TxtBgColor.Background = bgBrush
            Else
                Dim convertFromString As Object = ColorConverter.ConvertFromString("Black")
                If convertFromString IsNot Nothing Then
                    Dim bgColor = DirectCast(convertFromString, Media.Color)
                    TxtBgColor.Background = New SolidColorBrush(bgColor)
                End If
            End If

            ChkBorderStyle.IsChecked = Not String.IsNullOrEmpty(element.BorderStyle)
            CmbBorderStyle.Text = element.BorderStyle

            If ChkHeight.IsChecked = True AndAlso ChkWidth.IsChecked = True Then
                Try
                    Dim widthDigitPart As String
                    Dim unit As String
                    getValueAndUnit(TxtWidth.Text, widthDigitPart, unit)

                    Dim heightDigitPart As String
                    getValueAndUnit(TxtHeight.Text, heightDigitPart, unit)
                    If widthDigitPart.Length > 0 AndAlso heightDigitPart.Length > 0 Then
                        Dim width As Single = Single.Parse(widthDigitPart)
                        Dim height As Single = Single.Parse(heightDigitPart)
                        If width > 0 AndAlso height > 0 Then
                            Me._widthToHeightAspectRatio = width / height
                        Else
                            Me._widthToHeightAspectRatio = Nothing
                        End If
                    End If
                Catch
                    Me._widthToHeightAspectRatio = Nothing
                End Try
            Else
                Me._widthToHeightAspectRatio = Nothing
            End If
        End Sub

        ''' <summary>
        ''' Sets the image dimensiona and aspect ratio.
        ''' </summary>
        ''' <param name="imageFileName">Name of the image file.</param>
        Private Sub setImageDimensionaAndAspectRatio(imageFileName As String)
            Dim theImageDimension As Nullable(Of Size) = ImageUtils.GetImageDimension(imageFileName)
            If theImageDimension.HasValue Then
                ChkHeight.IsChecked = True
                TxtHeight.Text = theImageDimension.Value.Height.ToString(CultureInfo.InvariantCulture)
                ChkWidth.IsChecked = True
                TxtWidth.Text = theImageDimension.Value.Width.ToString(CultureInfo.InvariantCulture)
                If theImageDimension.Value.Width > 0 AndAlso theImageDimension.Value.Height > 0 Then
                    Me._widthToHeightAspectRatio = theImageDimension.Value.Width / CSng(theImageDimension.Value.Height)
                Else
                    Me._widthToHeightAspectRatio = Nothing
                End If
            Else
                Me._widthToHeightAspectRatio = Nothing
            End If
        End Sub

        ''' <summary>
        ''' Gets the value and unit.
        ''' </summary>
        ''' <param name="value">The value.</param>
        ''' <param name="digitPart">The digit part.</param>
        ''' <param name="unitPart">The unit part.</param>
        Private Shared Sub getValueAndUnit(value As String, ByRef digitPart As String, ByRef unitPart As String)
            Const digitRegEx = "\d+"
            digitPart = Regex.Match(value, digitRegEx, RegexOptions.IgnoreCase Or RegexOptions.Compiled).Groups(0).Value
            unitPart = value.Replace(digitPart, "").Trim()
        End Sub

        ''' <summary>
        ''' Gets or sets a value indicating whether this instance is local resource selection disabled.
        ''' </summary>
        ''' <value><c>true</c> if this instance is local resource selection disabled; otherwise, <c>false</c>.</value>
        Public Property IsLocalResourceSelectionDisabled() As Boolean Implements IImageDialog.IsLocalResourceSelectionDisabled
            Get
                Return m_IsLocalResourceSelectionDisabled
            End Get
            Set(value As Boolean)
                m_IsLocalResourceSelectionDisabled = value
            End Set
        End Property
        Private m_IsLocalResourceSelectionDisabled As Boolean

        Private Sub updateBrowserButtonState()
            BtnBrowseFile.IsEnabled = RdoLocalFile.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the OnRightButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub TwoButtonPanel_OnOnRightButtonClicked(sender As Object, e As RoutedEventArgs)
            Close()
        End Sub

        ''' <summary>
        ''' Handles the OnLeftButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub TwoButtonPanel_OnOnLeftButtonClicked(sender As Object, e As RoutedEventArgs)
            If String.IsNullOrEmpty(TxtUrl.Text) Then
                MessageBox.Show("Please provide Image Url")
                TxtUrl.Focus()
                Return
            End If

            DialogResult = True           
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkBtnBgColor control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="routedEventArgs">The evetn data</param>
        Private Sub lnkBtnBgColor_OnClick(sender As Object, routedEventArgs As RoutedEventArgs)
            Using colorDialog As IColorPickerDialog = _dialogService.ColorPickerDialog
                If colorDialog.ShowDialog() = True Then
                    TxtBgColor.Background = New SolidColorBrush(colorDialog.SelectedColor)
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Handles OnChecked event of the RdoInternetUrl RadioButton
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The event argument.</param>
        Private Sub RdoInternetUrl_OnChecked(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            ChkInsertLocalBase64.IsChecked = False
            ChkInsertLocalBase64.IsEnabled = False
        End Sub

        ''' <summary>
        ''' Handles OnUnchecked event of the RdoInternet control.
        ''' </summary>
        ''' <param name="sender">The event sender.</param>
        ''' <param name="e">The event sender.</param>
        Private Sub RdoInternet_OnUnchecked(sender As Object, e As RoutedEventArgs)
            ChkInsertLocalBase64.IsEnabled = True
        End Sub

        ''' <summary>
        ''' Shows this dialog.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function ShowDialog() As Boolean? Implements IDialogBase.ShowDialog
            Return MyBase.ShowDialog()
        End Function

        ''' <summary>
        ''' Gets and sets owner window for this window
        ''' </summary>
        Public Overloads Property Owner As Window Implements IDialog.Owner
            Get
                Return MyBase.Owner
            End Get
            Set(value As Window)
                MyBase.Owner = value
            End Set
        End Property

        Public Sub IDisposable_Dispose() Implements IDisposable.Dispose

        End Sub
    End Class
End Namespace