﻿
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media
Imports SpiceLogic.WpfHtmlEditor.Extensions
Imports SpiceLogic.WpfHtmlEditor.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.ToolbarModule

Namespace Dialogs.StyleBuilder

    ''' <summary>
    ''' Class ucFont
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Font", "font;font-family;font-size;text-decoration;font-weight;text-transform;color;font-style;font-variant")> _
    Partial Public Class ucFont
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

        ''' <summary>
        ''' A method creating the color dialog
        ''' </summary>
        Private ReadOnly _createColorDialogMethod As DialogFactoryDelegates.CreateColorPickerDialogDelegate

#Region "Preset of possible value"

        ''' <summary>
        ''' The _ system fonts
        ''' </summary>
        ReadOnly _SystemFonts As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ font style
        ''' </summary>
        ReadOnly _FontStyle As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ font variant
        ''' </summary>
        ReadOnly _FontVariant As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bold absolute
        ''' </summary>
        ReadOnly _BoldAbsolute As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bold relative
        ''' </summary>
        ReadOnly _BoldRelative As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ text transform
        ''' </summary>
        ReadOnly _TextTransform As New List(Of KeyValuePair(Of String, String))()
#End Region

        ''' <summary>
        ''' Creates the lists.
        ''' </summary>
        Protected Sub CreateLists()
            '#Region "Initialize presets"
            _SystemFonts.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Window caption", "caption"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("ToolWindow caption", "small-caption"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Dialog text", "message-box"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Icon labels", "icon"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Menu text", "menu"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Tooltip text", "status-bar"))

            _FontStyle.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _FontStyle.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _FontStyle.Add(New KeyValuePair(Of String, String)("Italic", "italic"))

            _FontVariant.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _FontVariant.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _FontVariant.Add(New KeyValuePair(Of String, String)("Small Caps", "small-caps"))

            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("Bold", "bold"))

            _BoldRelative.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _BoldRelative.Add(New KeyValuePair(Of String, String)("Lighter", "lighter"))
            _BoldRelative.Add(New KeyValuePair(Of String, String)("Bolder", "bolder"))

            _TextTransform.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _TextTransform.Add(New KeyValuePair(Of String, String)("None", "none"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("Initial Cap", "capitalize"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("lowercase", "lowercase"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("UPPERCASE", "uppercase"))
            '#End Region
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucFont"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        ''' <param name="createColorDialogMethod">A method creating the color dialog</param>
        Public Sub New(dict As Dictionary(Of String, String), createColorDialogMethod As DialogFactoryDelegates.CreateColorPickerDialogDelegate) 'the argument order is important (Activator.CreateInstance uses it)
            'the argument order is important (Activator.CreateInstance uses it)
            _dict = dict
            _createColorDialogMethod = createColorDialogMethod
            CreateLists()

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("font-family")
            _dict.Remove("font")
            _dict.Remove("color")
            _dict.Remove("font-style")
            _dict.Remove("font-variant")
            _dict.Remove("font-weight")
            _dict.Remove("font-size")
            _dict.Remove("text-decoration")

            If True Then
                ' handle text decoration
                Dim sb As New StringBuilder()
                If CbEffectNone.IsChecked = True Then
                    sb.Append(" none")
                End If
                If CbEffectUnderline.IsChecked = True Then
                    sb.Append(" underline")
                End If
                If CbEffectStrikethrough.IsChecked = True Then
                    sb.Append(" line-through")
                End If
                If CbEffectOverline.IsChecked = True Then
                    sb.Append(" overline")
                End If
                _dict("text-decoration") = sb.ToString()
            End If

            _dict("text-transform") = DirectCast(CbCapitalization.SelectedValue, String)

            If RbFamily.IsChecked = True Then
                _dict("font-family") = TbFontFamily.Text
                _dict("color") = WpfColorTranslator.ToHtml(DirectCast(TxtForeColor.Background, SolidColorBrush).Color)
                ' cbColor.Text.ToLowerInvariant();
                _dict("font-style") = DirectCast(CbFontStyle.SelectedValue, String)
                _dict("font-variant") = DirectCast(CbFontVariant.SelectedValue, String)

                ' font-size
                If RbSizeSpecific.IsChecked = True Then
                    If TbSpecificSize.Text.Trim().Length > 0 Then
                        _dict("font-size") = TbSpecificSize.Text + CbSpecificSizeType.Text
                    End If
                ElseIf RbSizeAbsolute.IsChecked = True Then
                    _dict("font-size") = CbAbsoluteSize.Text.ToLowerInvariant()
                Else
                    _dict("font-size") = CbRelativeSize.Text.ToLowerInvariant()
                End If

                ' font-weight
                If RbBoldAbsolute.IsChecked = True Then
                    _dict("font-weight") = DirectCast(CbBoldAbsolute.SelectedValue, String)
                Else
                    _dict("font-weight") = DirectCast(CbBoldRelative.SelectedValue, String)
                End If
            End If

            If RbSystemFont.IsChecked = True Then
                _dict("font") = DirectCast(CbSystemFont.SelectedValue, String)
            End If
        End Sub

        ''' <summary>
        ''' Handles the Loaded event of the ucFont control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub ucFont_Loaded(sender As Object, e As RoutedEventArgs)
            '#Region "set data sources"
            CbSystemFont.ItemsSource = _SystemFonts
            CbSystemFont.DisplayMemberPath = "Key"
            CbSystemFont.SelectedValuePath = "Value"
            CbSystemFont.SelectedIndex = 0

            CbFontStyle.ItemsSource = _FontStyle
            CbFontStyle.DisplayMemberPath = "Key"
            CbFontStyle.SelectedValuePath = "Value"
            CbFontStyle.SelectedIndex = 0

            CbFontVariant.ItemsSource = _FontVariant
            CbFontVariant.DisplayMemberPath = "Key"
            CbFontVariant.SelectedValuePath = "Value"
            CbFontVariant.SelectedIndex = 0

            CbBoldAbsolute.ItemsSource = _BoldAbsolute
            CbBoldAbsolute.DisplayMemberPath = "Key"
            CbBoldAbsolute.SelectedValuePath = "Value"
            CbBoldAbsolute.SelectedIndex = 0

            CbBoldRelative.ItemsSource = _BoldRelative
            CbBoldRelative.DisplayMemberPath = "Key"
            CbBoldRelative.SelectedValuePath = "Value"
            CbBoldRelative.SelectedIndex = 0

            CbCapitalization.ItemsSource = _TextTransform
            CbCapitalization.DisplayMemberPath = "Key"
            CbCapitalization.SelectedValuePath = "Value"
            CbCapitalization.SelectedIndex = 0
            '#End Region

            '#Region "set default values"
            RbFamily.IsChecked = True
            RbSizeSpecific.IsChecked = True
            RbBoldAbsolute.IsChecked = True
            '#End Region

            Dim value As String

            '#Region "parse"

            If _dict.TryGetValue("color", value) Then
                TxtForeColor.Background = New SolidColorBrush(WpfColorTranslator.FromHtml(value))
            End If

            If _dict.TryGetValue("font-style", value) Then
                Dim i As Integer = 0, n As Integer = _FontStyle.Count
                While i < n
                    If String.Equals(value, _FontStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbFontStyle.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("font-size", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = CbAbsoluteSize.Items.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, TryCast(TryCast(CbAbsoluteSize.Items(i), ComboBoxItem).Content, String), StringComparison.InvariantCultureIgnoreCase) Then
                        CbAbsoluteSize.SelectedIndex = i
                        RbSizeAbsolute.IsChecked = True
                        handled = True
                    End If
                    i += 1
                End While

                i = 0
                n = CbRelativeSize.Items.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, TryCast(TryCast(CbRelativeSize.Items(i), ComboBoxItem).Content, String), StringComparison.InvariantCultureIgnoreCase) Then
                        CbRelativeSize.SelectedIndex = i
                        RbSizeRelative.IsChecked = True
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    RbSizeSpecific.IsChecked = True
                    i = 0
                    n = CbSpecificSizeType.Items.Count
                    While i < n AndAlso Not handled
                        Dim cbSpecificSizeTypeItem As String = TryCast(TryCast(CbSpecificSizeType.Items(i), ComboBoxItem).Content, String)
                        If Not String.IsNullOrEmpty(cbSpecificSizeTypeItem) AndAlso value.EndsWith(cbSpecificSizeTypeItem, StringComparison.InvariantCultureIgnoreCase) Then
                            CbSpecificSizeType.SelectedIndex = i
                            TbSpecificSize.Text = value.Substring(0, value.Length - cbSpecificSizeTypeItem.Length)
                            handled = True
                        End If
                        i += 1
                    End While
                End If
            End If

            If _dict.TryGetValue("font-weight", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _BoldAbsolute.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, _BoldAbsolute(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbBoldAbsolute.SelectedIndex = i
                        RbBoldAbsolute.IsChecked = True
                        handled = True
                    End If
                    i += 1
                End While

                i = 0
                n = _BoldRelative.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, _BoldRelative(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbBoldRelative.SelectedIndex = i
                        RbBoldRelative.IsChecked = True
                        handled = True
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-decoration", value) Then
                Dim loValue As String = value.ToLowerInvariant()
                CbEffectUnderline.IsChecked = loValue.Contains("underline")
                CbEffectStrikethrough.IsChecked = loValue.Contains("line-through") OrElse loValue.Contains("linethrough")
                CbEffectOverline.IsChecked = loValue.Contains("overline")
                CbEffectNone.IsChecked = loValue.Contains("none")
            End If

            If _dict.TryGetValue("text-transform", value) Then
                Dim i As Integer = 0, n As Integer = _TextTransform.Count
                While i < n
                    If String.Equals(value, _TextTransform(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbCapitalization.SelectedIndex = i
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("font-family", value) Then
                RbFamily.IsChecked = True
                TbFontFamily.Text = value
            End If

            If _dict.TryGetValue("font", value) Then
                RbSystemFont.IsChecked = True

                Dim i As Integer = 0, n As Integer = _SystemFonts.Count
                While i < n
                    If String.Equals((If(value, "")).Replace("-", "").Trim(), _SystemFonts(i).Value.Replace("-", ""), StringComparison.InvariantCultureIgnoreCase) Then
                        CbSystemFont.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handlers"
        ''' <summary>
        ''' Handles the Click event of the BtFontFamilySelect control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub btFontFamilySelect_Click(sender As Object, e As RoutedEventArgs)
            Dim subForm As New FontPicker(TbFontFamily.Text)
            If subForm.ShowDialog() = True Then
                TbFontFamily.Text = subForm.SelectedFontList
            End If
        End Sub

        ''' <summary>
        ''' Fonts the type changed.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub FontTypeChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TbFontFamily.IsEnabled = InlineAssignHelper(btFontFamilySelect.IsEnabled, RbFamily.IsChecked.HasValue AndAlso RbFamily.IsChecked.Value = True)
            CbSystemFont.IsEnabled = RbFamily.IsChecked = False

            GbSize.IsEnabled = InlineAssignHelper(GbBold.IsEnabled, InlineAssignHelper(CbFontStyle.IsEnabled, InlineAssignHelper(CbFontVariant.IsEnabled, RbFamily.IsChecked.HasValue AndAlso RbFamily.IsChecked.Value = True)))
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the RbSizeSpecific control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeSpecific_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TbSpecificSize.IsEnabled = InlineAssignHelper(CbSpecificSizeType.IsEnabled, True)
            CbRelativeSize.IsEnabled = InlineAssignHelper(CbAbsoluteSize.IsEnabled, False)
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the RbSizeAbsolute control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeAbsolute_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CbAbsoluteSize.IsEnabled = True
            TbSpecificSize.IsEnabled = InlineAssignHelper(CbSpecificSizeType.IsEnabled, InlineAssignHelper(CbRelativeSize.IsEnabled, False))
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the RbSizeRelative control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeRelative_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CbRelativeSize.IsEnabled = True
            TbSpecificSize.IsEnabled = InlineAssignHelper(CbSpecificSizeType.IsEnabled, InlineAssignHelper(CbAbsoluteSize.IsEnabled, False))
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the cbEffectNone control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub cbEffectNone_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            If CbEffectNone.IsChecked = True Then
                CbEffectOverline.IsChecked = InlineAssignHelper(CbEffectStrikethrough.IsChecked, InlineAssignHelper(CbEffectUnderline.IsChecked, False))
                CbEffectOverline.IsEnabled = InlineAssignHelper(CbEffectStrikethrough.IsEnabled, InlineAssignHelper(CbEffectUnderline.IsEnabled, False))
            Else
                CbEffectOverline.IsEnabled = InlineAssignHelper(CbEffectStrikethrough.IsEnabled, InlineAssignHelper(CbEffectUnderline.IsEnabled, True))
            End If
        End Sub

        ''' <summary>
        ''' Rbs the bold radio button changed.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub rbBoldRadioButtonChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CbBoldAbsolute.IsEnabled = RbBoldAbsolute.IsChecked = True
            CbBoldRelative.IsEnabled = RbBoldRelative.IsChecked = True
        End Sub
#End Region


        ''' <summary>
        ''' Handles the PreviewMouseLeftButtonUp events of the TxtForeColor control.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub TxtForeColor_PreviewMouseLeftButtonDown(sender As Object, e As MouseButtonEventArgs)
            Dim solidColorBrush As SolidColorBrush = TryCast(TxtForeColor.Background, SolidColorBrush)
            If solidColorBrush IsNot Nothing Then
                Using colorDialog As IColorPickerDialog = _createColorDialogMethod()
                    colorDialog.StartingColor = solidColorBrush.Color
                    If colorDialog.ShowDialog() = True Then
                        TxtForeColor.Background = New SolidColorBrush(colorDialog.SelectedColor)
                    End If
                End Using
            End If
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class


End Namespace