﻿Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls

Namespace Dialogs.StyleBuilder

    ''' <summary>
    ''' Class ucPosition
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Position", "position,top,left,width,height,z-index")> _
    Partial Public Class ucPosition
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible values"

        ''' <summary>
        ''' The _ position
        ''' </summary>
        ReadOnly _position As New List(Of KeyValuePair(Of String, String))()
#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucPosition"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Initialize presets"
            _position.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _position.Add(New KeyValuePair(Of String, String)("Position in normal flow", "static"))
            _position.Add(New KeyValuePair(Of String, String)("Offset from normal flow", "relative"))
            _position.Add(New KeyValuePair(Of String, String)("Absolutely position", "absolute"))
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("position")
            _dict.Remove("top")
            _dict.Remove("left")
            _dict.Remove("width")
            _dict.Remove("height")
            _dict.Remove("z-index")

            If CbPositionMode.SelectedIndex >= 2 Then
                If TbLeft.Text.Trim().Length > 0 Then
                    _dict("left") = TbLeft.Text + CbLeftType.Text
                End If
                If TbTop.Text.Trim().Length > 0 Then
                    _dict("top") = TbTop.Text + CbTopType.Text
                End If

                If CbPositionMode.SelectedIndex = 3 Then
                    _dict("z-index") = TbZIndex.Text
                End If
            End If

            If TbHeight.Text.Trim().Length > 0 Then
                _dict("height") = TbHeight.Text + CbHeightType.Text
            End If
            If TbWidth.Text.Trim().Length > 0 Then
                _dict("width") = TbWidth.Text + CbWidthType.Text
            End If

            _dict("position") = DirectCast(CbPositionMode.SelectedValue, String)
        End Sub

        ''' <summary>
        ''' Handles the Loaded event of the ucPosition control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub ucPosition_Loaded(sender As Object, e As RoutedEventArgs)
            '#Region "set data sources"
            CbPositionMode.ItemsSource = _position
            CbPositionMode.DisplayMemberPath = "Key"
            CbPositionMode.SelectedValuePath = "Value"
            CbPositionMode.SelectedIndex = 0
            '#End Region

            Dim value As String

            '#Region "parse"
            If _dict.TryGetValue("left", value) Then
                Dim i As Integer = 0, n As Integer = CbLeftType.Items.Count
                While i < n
                    Dim leftType As String = TryCast(TryCast(CbLeftType.Items(i), ComboBoxItem).Content, String)
                    If value.EndsWith(leftType, StringComparison.InvariantCultureIgnoreCase) Then
                        TbLeft.Text = value.Substring(0, value.Length - leftType.Length)
                        CbLeftType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("top", value) Then
                Dim i As Integer = 0, n As Integer = CbTopType.Items.Count
                While i < n
                    Dim topType As String = TryCast(TryCast(CbTopType.Items(i), ComboBoxItem).Content, String)
                    If value.EndsWith(topType, StringComparison.InvariantCultureIgnoreCase) Then
                        TbTop.Text = value.Substring(0, value.Length - topType.Length)
                        CbTopType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("width", value) Then
                Dim i As Integer = 0, n As Integer = CbWidthType.Items.Count
                While i < n
                    Dim widthType As String = TryCast(TryCast(CbWidthType.Items(i), ComboBoxItem).Content, String)
                    If value.EndsWith(widthType, StringComparison.InvariantCultureIgnoreCase) Then
                        TbWidth.Text = value.Substring(0, value.Length - widthType.Length)
                        CbWidthType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("height", value) Then
                Dim i As Integer = 0, n As Integer = CbHeightType.Items.Count
                While i < n
                    Dim heightType As String = TryCast(TryCast(CbHeightType.Items(i), ComboBoxItem).Content, String)
                    If value.EndsWith(heightType, StringComparison.InvariantCultureIgnoreCase) Then
                        TbHeight.Text = value.Substring(0, value.Length - heightType.Length)
                        CbHeightType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("z-index", value) Then
                TbZIndex.Text = value
            End If

            If _dict.TryGetValue("position", value) Then
                Dim i As Integer = 0, n As Integer = _position.Count
                While i < n
                    If String.Equals(value, _position(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbPositionMode.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handlers"
        ''' <summary>
        ''' Handles the SelectionChanged event of the CbPositionMode control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub cbPositionMode_SelectionChanged(sender As Object, e As RoutedEventArgs)
            TbLeft.IsEnabled = InlineAssignHelper(CbLeftType.IsEnabled, InlineAssignHelper(TbTop.IsEnabled, InlineAssignHelper(CbTopType.IsEnabled, CbPositionMode.SelectedIndex >= 2)))

            TbZIndex.IsEnabled = CbPositionMode.SelectedIndex = 3
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
#End Region

    End Class
End Namespace