﻿
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls

Namespace Dialogs.StyleBuilder

    ''' <summary>
    ''' Class ucText
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Text", "text-align,vertical-align,text-justify,letter-spacing,line-height,direction,text-indent")> _
    Partial Public Class ucText
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible values"

        ''' <summary>
        ''' The _ text align
        ''' </summary>
        ReadOnly _textAlign As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ vertical align
        ''' </summary>
        ReadOnly _verticalAlign As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ text justify
        ''' </summary>
        ReadOnly _textJustify As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ letter spacing
        ''' </summary>
        ReadOnly _letterSpacing As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ line height
        ''' </summary>
        ReadOnly _lineHeight As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ direction
        ''' </summary>
        ReadOnly _direction As New List(Of KeyValuePair(Of String, String))()

#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucText"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Initialize presets"
            _textAlign.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _textAlign.Add(New KeyValuePair(Of String, String)("Left", "left"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Center", "center"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Right", "right"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Justified", "justify"))

            _verticalAlign.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("baseline", "baseline"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("sub", "sub"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("super", "super"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("top", "top"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("text-top", "text-top"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("middle", "middle"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("bottom", "bottom"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("text-bottom", "text-bottom"))

            _textJustify.Add(New KeyValuePair(Of String, String)("", ""))
            _textJustify.Add(New KeyValuePair(Of String, String)("Auto", "auto"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Space words", "inter-word"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Newspaper style", "newspaper"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Distribute spacing", "distribute"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Distribute all lines", "dibtribute-all-lines"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Inter-cluster", "inter-cluster"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Inter-ideograph", "inter-ideograph"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Kashida", "kashida"))

            _letterSpacing.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _letterSpacing.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _letterSpacing.Add(New KeyValuePair(Of String, String)("Custom", ""))

            _lineHeight.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _lineHeight.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _lineHeight.Add(New KeyValuePair(Of String, String)("Custom", ""))

            _direction.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _direction.Add(New KeyValuePair(Of String, String)("Left to right", "ltr"))
            _direction.Add(New KeyValuePair(Of String, String)("Right to left", "rtl"))
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            ' remove previous entries
            _dict.Remove("text-align")
            _dict.Remove("vertical-align")
            _dict.Remove("text-justify")

            _dict.Remove("letter-spacing")
            _dict.Remove("line-height")

            _dict.Remove("direction")
            _dict.Remove("text-indent")

            ' save form's data
            _dict("text-align") = DirectCast(CbAlHorizontal.SelectedValue, String)
            If CbAlHorizontal.SelectedIndex = 4 Then
                _dict("text-justify") = DirectCast(CbAlJustification.SelectedValue, String)
            End If
            _dict("vertical-align") = DirectCast(CbAlVertical.SelectedValue, String)

            _dict("letter-spacing") = If(CbSpacingLetters.SelectedIndex <> 2, DirectCast(CbSpacingLetters.SelectedValue, String), TbSpacingLetters.Text + CbSpacingLettersCustom.Text)

            _dict("line-height") = If(CbSpacingLines.SelectedIndex <> 2, DirectCast(CbSpacingLines.SelectedValue, String), TbSpacingLines.Text + CbSpacingLinesCustom.Text)

            If TbTextFlowIndentation.Text.Trim().Length > 0 Then
                _dict("text-indent") = TbTextFlowIndentation.Text + CbTextFlowCustom.Text
            End If

            _dict("direction") = DirectCast(CbTextFlowDirection.SelectedValue, String)
        End Sub

        ''' <summary>
        ''' Handles the Loaded event of the ucText control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub ucText_Loaded(sender As Object, e As RoutedEventArgs)
            '#Region "set data sources"
            CbAlHorizontal.ItemsSource = _textAlign
            CbAlHorizontal.DisplayMemberPath = "Key"
            CbAlHorizontal.SelectedValuePath = "Value"
            CbAlHorizontal.SelectedIndex = 0

            CbAlVertical.ItemsSource = _verticalAlign
            CbAlVertical.DisplayMemberPath = "Key"
            CbAlVertical.SelectedValuePath = "Value"
            CbAlVertical.SelectedIndex = 0

            CbAlJustification.ItemsSource = _textJustify
            CbAlJustification.DisplayMemberPath = "Key"
            CbAlJustification.SelectedValuePath = "Value"
            CbAlJustification.SelectedIndex = 0

            CbSpacingLetters.ItemsSource = _letterSpacing
            CbSpacingLetters.DisplayMemberPath = "Key"
            CbSpacingLetters.SelectedValuePath = "Value"
            CbSpacingLetters.SelectedIndex = 0

            CbSpacingLines.ItemsSource = _lineHeight
            CbSpacingLines.DisplayMemberPath = "Key"
            CbSpacingLines.SelectedValuePath = "Value"
            CbSpacingLines.SelectedIndex = 0

            CbTextFlowDirection.ItemsSource = _direction
            CbTextFlowDirection.DisplayMemberPath = "Key"
            CbTextFlowDirection.SelectedValuePath = "Value"
            CbTextFlowDirection.SelectedIndex = 0
            '#End Region

            Dim value As String

            '#Region "parse alignment"
            If _dict.TryGetValue("vertical-align", value) Then
                Dim i As Integer = 0, n As Integer = _verticalAlign.Count
                While i < n
                    If String.Equals(value, _verticalAlign(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbAlVertical.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-justify", value) Then
                Dim i As Integer = 0, n As Integer = _textJustify.Count
                While i < n
                    If String.Equals(value, _textJustify(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbAlJustification.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-align", value) Then
                Dim i As Integer = 0, n As Integer = _textAlign.Count
                While i < n
                    If String.Equals(value, _textAlign(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbAlHorizontal.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region

            '#Region "parse spacing"
            If _dict.TryGetValue("letter-spacing", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _letterSpacing.Count
                While i < n AndAlso Not handled
                    If value.Equals(_letterSpacing(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbSpacingLetters.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    i = 0
                    n = CbSpacingLettersCustom.Items.Count
                    While i < n
                        Dim lettersCustom As String = TryCast(TryCast(CbSpacingLettersCustom.Items(i), ComboBoxItem).Content, String)
                        If value.EndsWith(lettersCustom, StringComparison.InvariantCultureIgnoreCase) Then
                            TbSpacingLetters.Text = value.Substring(0, value.Length - lettersCustom.Length)
                            CbSpacingLettersCustom.SelectedIndex = i
                            CbSpacingLetters.SelectedIndex = 2
                            Exit While
                        End If
                        i += 1
                    End While
                End If
            End If

            If _dict.TryGetValue("line-height", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _lineHeight.Count
                While i < n AndAlso Not handled
                    If value.Equals(_lineHeight(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbSpacingLines.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    i = 0
                    n = CbSpacingLinesCustom.Items.Count
                    While i < n
                        Dim linesCustom As String = TryCast(TryCast(CbSpacingLinesCustom.Items(i), ComboBoxItem).Content, String)
                        If value.EndsWith(linesCustom, StringComparison.InvariantCultureIgnoreCase) Then
                            TbSpacingLines.Text = value.Substring(0, value.Length - linesCustom.Length)
                            CbSpacingLinesCustom.SelectedIndex = i
                            CbSpacingLines.SelectedIndex = 2
                            Exit While
                        End If
                        i += 1
                    End While
                End If
            End If
            '#End Region

            '#Region "parse text flow"
            If _dict.TryGetValue("text-indent", value) Then
                Dim i As Integer = 0, n As Integer = CbTextFlowCustom.Items.Count
                While i < n
                    Dim textFlowCustom As String = TryCast(TryCast(CbTextFlowCustom.Items(i), ComboBoxItem).Content, String)
                    If value.EndsWith(textFlowCustom, StringComparison.InvariantCultureIgnoreCase) Then
                        TbTextFlowIndentation.Text = value.Substring(0, value.Length - textFlowCustom.Length)
                        CbTextFlowCustom.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("direction", value) Then
                Dim i As Integer = 0, n As Integer = _direction.Count
                While i < n
                    If value.Equals(_direction(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        CbTextFlowDirection.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handling"
        ''' <summary>
        ''' Handles the SelectionChanged event of the CbAlHorizontal control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub cbAlHorizontal_SelectionChanged(sender As Object, e As RoutedEventArgs)
            CbAlJustification.IsEnabled = CbAlHorizontal.SelectedIndex = 4
        End Sub

        ''' <summary>
        ''' Handles the SelectionChanged event of the cbSpacingLetters control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub cbSpacingLetters_SelectionChanged(sender As Object, e As RoutedEventArgs)
            TbSpacingLetters.IsEnabled = InlineAssignHelper(CbSpacingLettersCustom.IsEnabled, CbSpacingLetters.SelectedIndex = 2)
        End Sub

        ''' <summary>
        ''' Handles the SelectionChanged event of the cbSpacingLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub cbSpacingLines_SelectionChanged(sender As Object, e As RoutedEventArgs)
            TbSpacingLines.IsEnabled = InlineAssignHelper(CbSpacingLinesCustom.IsEnabled, CbSpacingLines.SelectedIndex = 2)
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
#End Region

    End Class
End Namespace