﻿
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.Globalization
Imports System.Windows
Imports System.Windows.Input
Imports System.Windows.Media
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Services
Imports SpiceLogic.WpfHtmlEditor.Extensions
Imports Color = System.Drawing.Color
Imports ColorConverter = System.Drawing.ColorConverter

Namespace Dialogs

    ''' <summary>
    ''' Class TableCellPropertiesDialog
    ''' </summary>
    Partial Public Class TableCellPropertiesDialog
        Implements ITableCellDialog

        ''' <summary>
        ''' The _element
        ''' </summary>
        Private _element As TableCellElement

        ''' <summary>
        ''' The dialog service
        ''' </summary>
        Private ReadOnly _dialogService As IDialogService

        Private ReadOnly _propertiesAffected As New List(Of String)() From { _
            "BgColor", _
            "CSSClassName", _
            "Width", _
            "WidthUnit", _
            "Height", _
            "HeightUnit", _
            "HorizontalAlign", _
            "VerticalAlign", _
            "NoWrap" _
        }

        ''' <summary>
        ''' Initializes a new instance of the <see cref="TableCellPropertiesDialog" /> class.
        ''' </summary>
        ''' <param name="dialogService">The dialog service.</param>
        Public Sub New(dialogService As IDialogService)
            _dialogService = dialogService

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As TableCellElement Implements SpiceLogic.HtmlEditorControl.Domain.Dialogs.ITableCellDialogBase.Element
            Get
                _element.ResetStyles(Me._propertiesAffected)

                '                _element.BgColor = chkBgColor.Checked ? ColorTranslator.ToHtml(txtBgColor.BackColor) : null;


                If ChkBgColor.IsChecked = True Then
                    Dim backgroundColorBrush As SolidColorBrush = DirectCast(TxtBgColor.Background, SolidColorBrush)
                    _element.BgColor = WpfColorTranslator.ToHtml(backgroundColorBrush.Color)
                Else
                    _element.BgColor = Nothing
                End If


                _element.CSSClassName = TxtClassName.Text.Trim()
                _element.CSSStyle = TxtCss.Text.Trim()

                If ChkWidth.IsChecked = True AndAlso CmbWidthUnit.SelectedItem IsNot Nothing Then
                    _element.Width = Convert.ToInt32(TxtWidth.Text.Trim())
                    _element.WidthUnit = CmbWidthUnit.Text
                End If

                If ChkHeight.IsChecked = True AndAlso CmbHeightUnit.SelectedItem IsNot Nothing Then
                    _element.Height = Convert.ToInt32(TxtHeight.Text.Trim())
                    _element.HeightUnit = CmbHeightUnit.Text
                End If

                If CmbHorizontalAlign.SelectedItem IsNot Nothing Then
                    _element.HorizontalAlign = CmbHorizontalAlign.Text
                End If

                If CmbVerticalAlign.SelectedItem IsNot Nothing Then
                    _element.VerticalAlign = CmbVerticalAlign.Text
                End If
                _element.NoWrap = ChkNoWrap.IsChecked = True
                _element.OverrideSettingsToAllCells = ChkOverrideSettings4Cells.IsChecked = True
                Return _element
            End Get
            Set(value As TableCellElement)
                _element = value

                If _element Is Nothing Then
                    Return
                End If

                ChkBgColor.IsChecked = Not String.IsNullOrEmpty(_element.BgColor)
                If ChkBgColor.IsChecked = True Then
                    Dim converter As New ColorConverter()
                    Dim convertFromString As Object = converter.ConvertFromString(_element.BgColor)
                    If convertFromString IsNot Nothing Then
                        Dim backgroundDrawingColor As Color = DirectCast(convertFromString, Color)
                        Dim backgroundColor As System.Windows.Media.Color = System.Windows.Media.Color.FromArgb(backgroundDrawingColor.A, backgroundDrawingColor.R, backgroundDrawingColor.G, backgroundDrawingColor.B)
                        TxtBgColor.Background = New SolidColorBrush(backgroundColor)
                    End If
                End If

                TxtClassName.Text = _element.CSSClassName
                ChkWidth.IsChecked = _element.Width.HasValue
                If ChkWidth.IsChecked = True AndAlso _element.WidthUnit IsNot Nothing Then
                    If _element.Width.HasValue Then
                        TxtWidth.Text = _element.Width.Value.ToString(CultureInfo.InvariantCulture)
                    End If
                    CmbWidthUnit.Text = _element.WidthUnit
                End If

                ChkHeight.IsChecked = _element.Height.HasValue
                If ChkHeight.IsChecked = True AndAlso _element.HeightUnit IsNot Nothing Then
                    If _element.Height.HasValue Then
                        TxtHeight.Text = _element.Height.Value.ToString(CultureInfo.InvariantCulture)
                    End If
                    CmbHeightUnit.Text = _element.HeightUnit
                End If

                If _element.HorizontalAlign IsNot Nothing Then
                    CmbHorizontalAlign.Text = _element.HorizontalAlign
                End If

                If _element.VerticalAlign IsNot Nothing Then
                    CmbVerticalAlign.Text = _element.VerticalAlign
                End If
                ChkNoWrap.IsChecked = _element.NoWrap
                ChkOverrideSettings4Cells.IsChecked = _element.OverrideSettingsToAllCells

                TxtCss.Text = _element.GetCssStyleWithoutProperties(Me._propertiesAffected)
            End Set
        End Property
        ''' <summary>
        ''' Gets the cell attribute collection.
        ''' </summary>
        ''' <value>The cell attribute collection.</value>
        Public ReadOnly Property CellAttributeCollection() As NameValueCollection
            Get
                Dim myColl As New NameValueCollection()

                If ChkWidth.IsChecked = True Then
                    myColl.Add("width", String.Concat(TxtWidth.Text.Trim(), CmbWidthUnit.Text.Trim()))
                End If

                If ChkHeight.IsChecked = True Then
                    myColl.Add("height", String.Concat(TxtHeight.Text.Trim(), CmbHeightUnit.Text.Trim()))
                End If

                If ChkBgColor.IsChecked = True Then
                    Dim solidColorBrush As SolidColorBrush = DirectCast(TxtBgColor.Background, SolidColorBrush)
                    myColl.Add("bgcolor", WpfColorTranslator.ToHtml(solidColorBrush.Color))
                End If

                If CmbHorizontalAlign.SelectedIndex <> 0 Then
                    myColl.Add("align", CmbHorizontalAlign.Text)
                End If

                If CmbVerticalAlign.SelectedIndex <> 0 Then
                    myColl.Add("valign", CmbVerticalAlign.Text)
                End If

                If ChkNoWrap.IsChecked = True Then
                    myColl.Add("nowrap", "nowrap")
                End If

                Return myColl
            End Get
        End Property

        ''' <summary>
        ''' Gets the table cell attribute string.
        ''' </summary>
        ''' <value>The table cell attribute string.</value>
        Public ReadOnly Property TableCellAttributeString() As String
            Get
                Dim tableCellAttributes As String() = New String(-1) {}
                Dim tableCellStyleAttributes As String() = New String(-1) {}

                If ChkWidth.IsChecked = True Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("width: {0}{1}", TxtWidth.Text.Trim(), CmbWidthUnit.Text.Trim())
                End If

                If ChkHeight.IsChecked = True Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("height: {0}{1}", TxtHeight.Text.Trim(), CmbHeightUnit.Text.Trim())
                End If

                If ChkBgColor.IsChecked = True Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    Dim solidBackgroundBrush As SolidColorBrush = DirectCast(TxtBgColor.Background, SolidColorBrush)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("background-color: {0}", WpfColorTranslator.ToHtml(solidBackgroundBrush.Color))
                End If

                If tableCellStyleAttributes.Length <> 0 Then
                    Dim tableCellStyleText As String = String.Format("style = ""{0}""", String.Join("; ", tableCellStyleAttributes))
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = tableCellStyleText
                End If

                If CmbHorizontalAlign.SelectedIndex <> 0 Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = String.Format("align=""{0}""", CmbHorizontalAlign.Text)
                End If

                If CmbVerticalAlign.SelectedIndex <> 0 Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = String.Format("valign=""{0}""", CmbVerticalAlign.Text)
                End If

                If ChkNoWrap.IsChecked = True Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = "nowrap=""nowrap"""
                End If

                If Not String.IsNullOrEmpty(TxtClassName.Text.Trim()) Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = "class=""" + TxtClassName.Text.Trim() + """"
                End If

                Return String.Join(" ", tableCellAttributes)
            End Get
        End Property

        ''' <summary>
        ''' Get a lock for a property [override cell attributes].
        ''' </summary>
        Public Function LockOverrideSettingsToAllCells() As IDisposable Implements SpiceLogic.HtmlEditorControl.Domain.Dialogs.ITableCellDialogBase.LockOverrideSettingsToAllCells
            Return New lockOverrideSettingsToAllCellsClass(Me)
        End Function

        Public Sub Dispose() Implements IDisposable.Dispose
        End Sub


        ''' <summary>
        ''' Gets a value indicating whether [override cell attributes].
        ''' </summary>
        ''' <value><c>true</c> if [override cell attributes]; otherwise, <c>false</c>.</value>
        Public ReadOnly Property OverrideCellAttributes() As Boolean
            Get
                Return ChkOverrideSettings4Cells.IsChecked = True
            End Get
        End Property

#Region "--------------- UI Event Handlers ------------------"

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkWidth_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtWidth.IsEnabled = ChkWidth.IsChecked = True
            CmbWidthUnit.IsEnabled = ChkWidth.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkHeight control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkHeight_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtHeight.IsEnabled = ChkHeight.IsChecked = True
            CmbHeightUnit.IsEnabled = ChkHeight.IsChecked = True
        End Sub

#End Region
        Private Class lockOverrideSettingsToAllCellsClass
            Implements IDisposable

            Private ReadOnly _dialog As TableCellPropertiesDialog
            Private ReadOnly _initallyEnabled As Boolean

            Public Sub New(dialog As TableCellPropertiesDialog)
                _dialog = dialog
                _initallyEnabled = _dialog.ChkOverrideSettings4Cells.IsEnabled
                _dialog.ChkOverrideSettings4Cells.IsEnabled = False
            End Sub

            Public Sub Dispose() Implements IDisposable.Dispose
                _dialog.ChkOverrideSettings4Cells.IsEnabled = _initallyEnabled
            End Sub
        End Class


        ''' <summary>
        ''' Handles the Click event of the LnkBtnBgColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="routedEventArgs">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub lnkBtnBgColor_Click(sender As Object, routedEventArgs As RoutedEventArgs)
            showDialogForChoosingColor()
        End Sub

        Private Sub showDialogForChoosingColor()
            Dim bgColorBrush As SolidColorBrush = TryCast(TxtBgColor.Background, SolidColorBrush)
            Dim color As New System.Windows.Media.Color()

            If bgColorBrush IsNot Nothing Then
                color = bgColorBrush.Color
            Else
                Dim convertFromString As Object = System.Windows.Media.ColorConverter.ConvertFromString("Black")
                If convertFromString IsNot Nothing Then
                    color = DirectCast(convertFromString, System.Windows.Media.Color)
                End If
            End If

            Using colorDialog As IColorPickerDialog = _dialogService.ColorPickerDialog
                colorDialog.StartingColor = color

                If colorDialog.ShowDialog() = True Then
                    TxtBgColor.Background = New SolidColorBrush(colorDialog.SelectedColor)
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Handles the OnLeftButtonClicked event of the TwoButtonPanel control.
        ''' </summary>
        ''' <param name="sender">The event source.</param>
        ''' <param name="e">The event data.</param>
        Private Sub TwoButtonPanel_OnLeftButtonClicked(sender As Object, e As RoutedEventArgs)
           'ok button

            DialogResult = True           
        End Sub

        ''' <summary>
        ''' Handles the OnRightButtonClicked event of the TwoButtonPanel control.
        ''' </summary>
        ''' <param name="sender">The event source.</param>
        ''' <param name="e">The event data.</param>
        Private Sub TwoButtonPanel_OnRightButtonClicked(sender As Object, e As RoutedEventArgs)
            'cancel

            Close()
        End Sub

        ''' <summary>
        ''' Handles the MouseLeftButtonDown event of the DialogHeader control.
        ''' </summary>
        ''' <param name="sender">The event source.</param>
        ''' <param name="e">The event data.</param>
        Private Sub DialogHeader_MouseLeftButtonDown(sender As Object, e As MouseButtonEventArgs)
            DragMove()
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBgColor control.
        ''' </summary>
        ''' <param name="sender">The event source.</param>
        ''' <param name="e">The event data.</param>
        Private Sub chkBgColor_OnCheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            LnkBtnBgColor.IsEnabled = ChkBgColor.IsChecked = True
        End Sub


        ''' <summary>
        ''' Gets and sets owner window for this window
        ''' </summary>
        Public Overloads Property Owner() As Window Implements IDialog.Owner
            Get
                Return MyBase.Owner
            End Get
            Set(value As Window)
                MyBase.Owner = value
            End Set
        End Property

        ''' <summary>
        ''' Shows this window as a dialog
        ''' </summary>
        ''' <returns>Returns <c>true</c> if closed using button OK</returns>
        Public Overloads Function ShowDialog() As Boolean? Implements IDialogBase.ShowDialog
            Return MyBase.ShowDialog()
        End Function

    End Class
End Namespace