﻿
Imports System.Collections.Generic
Imports System.Globalization
Imports System.Windows
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports Microsoft.Win32
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Services
Imports SpiceLogic.WpfHtmlEditor.Extensions
Imports Color = System.Drawing.Color
Imports ColorConverter = System.Drawing.ColorConverter

Namespace Dialogs

    ''' <summary>
    ''' Class TablePropertiesDialog
    ''' </summary>
    Partial Public Class TablePropertiesDialog
        Implements ITableDialog

        ''' <summary>
        ''' The _element
        ''' </summary>
        Private _element As TableElement

        ''' <summary>
        ''' The _background picture URL
        ''' </summary>
        Private _backgroundPictureUrl As String = String.Empty

        Private ReadOnly _propertiesAffected As New List(Of String)() From { _
            "Rows", _
            "Columns", _
            "Width", _
            "Height", _
            "Caption", _
            "BorderWidth", _
            "CellPadding", _
            "CellSpacing", _
            "BorderColor", _
            "BorderAttr", _
            "BorderStyle", _
            "BgColor", _
            "BorderCollapse", _
            "SummaryDescription", _
            "ID", _
            "Name", _
            "CSSClassName", _
            "BackGround" _
        }

        ''' <summary>
        ''' The dialog service
        ''' </summary>
        Private ReadOnly _dialogService As IDialogService

        ''' <summary>
        ''' Initializes a new instance of the <see cref="TablePropertiesDialog" /> class.
        ''' </summary>
        ''' <param name="dialogService">The dialog service.</param>
        Public Sub New(dialogService As IDialogService)
            _dialogService = dialogService

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Handles the Loaded event of the TablePropertiesDialog control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        Private Sub TablePropertiesDialog_Loaded(sender As Object, e As RoutedEventArgs)
            BtnCellProperties.Visibility = If(Me.getOrInitCellElement() IsNot Nothing, Visibility.Visible, Visibility.Hidden)
            TxtId.Visibility = Visibility.Collapsed
            TxtName.Visibility = Visibility.Collapsed
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As TableElement Implements SpiceLogic.HtmlEditorControl.Domain.Dialogs.ITableDialogBase.Element
            Get
                If _element Is Nothing Then
                    _element = New TableElement(Nothing)
                End If

                _element.ResetValues(Me._propertiesAffected)

                _element.Rows = CInt(NumRows.Value)
                _element.Columns = CInt(NumCols.Value)

                If ChkCellPadding.IsChecked = True Then
                    _element.CellPadding = CInt(NumCellPadding.Value)
                End If
                If ChkCellSpacing.IsChecked = True Then
                    _element.CellSpacing = CInt(NumCellSpacing.Value)
                End If

                If ChkWidth.IsChecked = True AndAlso CmbWidthUnit.SelectedItem IsNot Nothing Then
                    _element.Width = Convert.ToInt32(TxtWidth.Text.Trim())
                    _element.WidthUnit = CmbWidthUnit.Text
                End If

                If ChkHeight.IsChecked = True AndAlso CmbHeightUnit.SelectedItem IsNot Nothing Then
                    _element.Height = Convert.ToInt32(TxtHeight.Text.Trim())
                    _element.HeightUnit = CmbHeightUnit.Text
                End If

                If ChkCaption.IsChecked = True Then
                    _element.Caption = TxtCaption.Text.Trim()
                End If

                If ChkBorderWidth.IsChecked = True Then
                    If ChkBorderStyle.IsChecked = True AndAlso CmbBorderStyle.SelectedItem IsNot Nothing Then
                        _element.BorderStyle = CmbBorderStyle.Text
                        _element.BorderWidth = CInt(NumBorderWidth.Value)
                    Else
                        _element.BorderAttr = CInt(NumBorderWidth.Value)
                    End If
                End If

                If LnkButtonBorderColor.IsEnabled AndAlso ChkBorderColor.IsChecked = True Then
                    Dim brush As SolidColorBrush = TryCast(TxtBorderColor.Background, SolidColorBrush)
                    If brush IsNot Nothing Then
                        _element.BorderColor = WpfColorTranslator.ToHtml(brush.Color)
                    End If
                End If

                If ChkBgColor.IsChecked = True Then
                    Dim brush As SolidColorBrush = TryCast(TxtBgColor.Background, SolidColorBrush)
                    If brush IsNot Nothing Then
                        _element.BgColor = WpfColorTranslator.ToHtml(brush.Color)
                    End If
                End If

                _element.SummaryDescription = TxtSummaryDescription.Text
                _element.BorderCollapse = ChkBorderCollapse.IsEnabled AndAlso ChkBorderCollapse.IsChecked = True
                _element.ID = TxtId.Text.Trim()
                _element.Name = TxtName.Text.Trim()
                _element.CSSClassName = TxtClassName.Text.Trim()
                _element.CSSStyle = TxtCss.Text.Trim()

                If ChkBackgroundPicture.IsChecked = True Then
                    _element.BackGround = ImgBackgroundPicture.Source.ToString()
                End If

                _element.BorderToAll = ChkBorderToAll.IsChecked = True

                Return _element
            End Get
            Set(value As TableElement)
                _element = value
                If _element Is Nothing Then
                    Return
                End If

                NumRows.Value = _element.Rows
                NumCols.Value = _element.Columns

                ChkWidth.IsChecked = _element.Width.HasValue

                If ChkWidth.IsChecked = True Then
                    TxtWidth.Text = If(_element.Width.HasValue, _element.Width.Value.ToString(CultureInfo.InvariantCulture), "")
                    CmbWidthUnit.SelectedItem = _element.WidthUnit
                End If

                ChkHeight.IsChecked = _element.Height.HasValue

                If ChkHeight.IsChecked = True Then
                    TxtHeight.Text = If(_element.Height.HasValue, _element.Height.Value.ToString(CultureInfo.InvariantCulture), "")
                    CmbHeightUnit.SelectedItem = _element.HeightUnit
                End If

                ChkCaption.IsChecked = Not String.IsNullOrEmpty(_element.Caption)

                If ChkCaption.IsChecked = True Then
                    TxtCaption.Text = _element.Caption
                End If

                ChkBorderWidth.IsChecked = _element.BorderWidth.HasValue OrElse _element.BorderAttr.HasValue

                ChkBorderStyle.IsChecked = Not String.IsNullOrEmpty(_element.BorderStyle)
                CmbBorderStyle.IsEnabled = ChkBorderStyle.IsChecked = True
                ChkBorderCollapse.IsEnabled = ChkBorderStyle.IsChecked = True
                ChkBorderColor.IsEnabled = ChkBorderStyle.IsChecked = True

                If ChkBorderStyle.IsChecked = True Then
                    CmbBorderStyle.Text = _element.BorderStyle
                    If ChkBorderWidth.IsChecked = True Then
                        NumBorderWidth.Value = If(_element.BorderWidth, 0)
                    End If
                Else
                    If ChkBorderWidth.IsChecked = True Then
                        NumBorderWidth.Value = If(_element.BorderAttr, (If(_element.BorderWidth, 0)))
                    End If
                End If

                ChkCellPadding.IsChecked = _element.CellPadding.HasValue

                If ChkCellPadding.IsChecked = True Then
                    NumCellPadding.Value = If(_element.CellPadding, 0)
                End If

                ChkCellSpacing.IsChecked = _element.CellSpacing.HasValue
                If ChkCellSpacing.IsChecked = True Then
                    If _element.CellSpacing IsNot Nothing Then
                        NumCellSpacing.Value = _element.CellSpacing.Value
                    End If
                    ChkCellSpacing.IsChecked = True
                End If

                ChkBorderColor.IsChecked = Not String.IsNullOrEmpty(_element.BorderColor)
                If ChkBorderColor.IsChecked = True Then
                    Dim converter As New ColorConverter()
                    Dim drawingColor As Color = DirectCast(converter.ConvertFromString(_element.BorderColor), Color)
                    Dim borderColor As System.Windows.Media.Color = System.Windows.Media.Color.FromArgb(drawingColor.A, drawingColor.R, drawingColor.G, drawingColor.B)
                    TxtBorderColor.Background = New SolidColorBrush(borderColor)
                End If

                ChkBgColor.IsChecked = InlineAssignHelper(LnkBtnBackgroundPicture.IsEnabled, Not String.IsNullOrEmpty(_element.BgColor))


                If ChkBgColor.IsChecked = True Then
                    Dim converter As New ColorConverter()
                    Dim backgroundColor As Color = DirectCast(converter.ConvertFromString(_element.BgColor), Color)
                    Dim color As System.Windows.Media.Color = System.Windows.Media.Color.FromArgb(backgroundColor.A, backgroundColor.R, backgroundColor.G, backgroundColor.B)
                    TxtBgColor.Background = New SolidColorBrush(color)
                End If

                ChkBorderCollapse.IsChecked = _element.BorderCollapse IsNot Nothing
                If ChkBorderCollapse.IsChecked = True Then
                    ChkBorderCollapse.IsChecked = _element.BorderCollapse.Value
                End If
                TxtSummaryDescription.Text = _element.SummaryDescription
                TxtId.Text = _element.ID
                TxtName.Text = _element.Name
                TxtClassName.Text = _element.CSSClassName
                '''//////////////
                ChkBackgroundPicture.IsChecked = Not String.IsNullOrEmpty(_element.BackGround)

                If ChkBackgroundPicture.IsChecked = True Then
                    ImgBackgroundPicture.Source = New BitmapImage(New Uri(_element.BackGround))
                End If

                TxtCss.Text = _element.GetCssStyleWithoutProperties(Me._propertiesAffected)
                ChkBorderToAll.IsChecked = True
            End Set
        End Property

#Region "------------ UI Event Handlers --------------"

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkWidth_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtWidth.IsEnabled = ChkWidth.IsChecked = True
            CmbWidthUnit.IsEnabled = ChkWidth.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkHeight control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkHeight_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtHeight.IsEnabled = ChkHeight.IsChecked = True
            CmbHeightUnit.IsEnabled = ChkHeight.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderWidth_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            NumBorderWidth.IsEnabled = ChkBorderWidth.IsChecked = True
            ChkBorderStyle.IsEnabled = ChkBorderWidth.IsChecked = True

            '            chkBorderStyle.Checked = chkBorderWidth.Checked;
            CmbBorderStyle.IsEnabled = ChkBorderWidth.IsChecked = True AndAlso ChkBorderStyle.IsChecked = True
            If ChkBorderWidth.IsChecked = False Then
                ChkBorderToAll.IsChecked = True
            End If
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBgColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBgColor_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            LnkButtonBgColor.IsEnabled = InlineAssignHelper(TxtBgColor.IsEnabled, ChkBgColor.IsChecked.HasValue AndAlso ChkBgColor.IsChecked.Value = True)
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderColor_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            LnkButtonBorderColor.IsEnabled = InlineAssignHelper(TxtBorderColor.IsEnabled, ChkBorderColor.IsChecked.HasValue AndAlso ChkBorderColor.IsChecked.Value = True)
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkCellPadding control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkCellPadding_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            NumCellPadding.IsEnabled = ChkCellPadding.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkCellSpacing control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkCellSpacing_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            NumCellSpacing.IsEnabled = ChkCellSpacing.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkBtnBackgroundPicture control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="routedEventArgs">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub lnkBtnBackgroundPicture_Click(sender As Object, routedEventArgs As RoutedEventArgs)
            Try
                Dim srcUrlDialog As New OpenFileDialog() With { _
                    .Title = "Please Select an image file.", _
                    .RestoreDirectory = True, _
                    .Filter = "Image Files|*.png;*.bmp;*.gif;*.jpg|All files(*.*)|*.*", _
                    .FilterIndex = 0, _
                    .Multiselect = False _
                }

                If srcUrlDialog.ShowDialog() = True Then
                    _backgroundPictureUrl = srcUrlDialog.FileName
                    ImgBackgroundPicture.Source = New BitmapImage(New Uri(_backgroundPictureUrl))
                End If
            Catch
            End Try
        End Sub

        ''' <summary>
        ''' Opens a window and returns only when the newly opened window is closed
        ''' </summary>
        ''' <returns></returns>
        Public Shadows Function ShowDialog() As System.Nullable(Of Boolean) Implements IDialogBase.ShowDialog
            Return MyBase.ShowDialog()
        End Function

        ''' <summary>
        ''' Handles the Click event of the btnCellProperties control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub btnCellProperties_Click(sender As Object, e As RoutedEventArgs)
            Dim tblCellElement As TableCellElement = Me.getOrInitCellElement()
            If tblCellElement Is Nothing Then
                MessageBox.Show("No cells were found.", "Error")
                Return
            End If

            Using tableCellDialog As ITableCellDialog = _dialogService.TableCellDialog
                tableCellDialog.Element = tblCellElement
                Using tableCellDialog.LockOverrideSettingsToAllCells()
                    If tableCellDialog.ShowDialog() = True Then
                        Me.Element.CellElement = tableCellDialog.Element
                    End If
                End Using
            End Using
        End Sub

        Private Function getOrInitCellElement() As TableCellElement
            Return Me.Element.GetFirstCellElement()
        End Function

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkCaption control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkCaption_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            TxtCaption.IsEnabled = ChkCaption.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBorderStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBorderStyle_CheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            CmbBorderStyle.IsEnabled = ChkBorderStyle.IsChecked = True
            ChkBorderCollapse.IsEnabled = ChkBorderStyle.IsChecked = True
            ChkBorderColor.IsEnabled = ChkBorderStyle.IsChecked = True
            If ChkBorderStyle.IsChecked = False Then
                ChkBorderToAll.IsChecked = True
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the NumBorderWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedPropertyChangedEventArgs{T}" /> instance containing the event data.</param>
        Private Sub numBorderWidth_ValueChanged(sender As Object, e As RoutedPropertyChangedEventArgs(Of Double))
            If Math.Abs(NumBorderWidth.Value) < 0.01 Then
                ChkBorderToAll.IsChecked = True
            End If
        End Sub

#End Region

        Public Sub Dispose() Implements IDisposable.Dispose
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkButtonBorderColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub lnkButtonBorderColor_OnClick(sender As Object, e As RoutedEventArgs)
            Dim backgroundBrush As SolidColorBrush = TryCast(TxtBorderColor.Background, SolidColorBrush)
            Dim color As New System.Windows.Media.Color()
            If backgroundBrush IsNot Nothing Then
                color = backgroundBrush.Color
            Else
                Dim convertFromString As Object = System.Windows.Media.ColorConverter.ConvertFromString("Black")
                If convertFromString IsNot Nothing Then
                    color = DirectCast(convertFromString, System.Windows.Media.Color)
                End If
            End If

            Using colorDialog As IColorPickerDialog = _dialogService.ColorPickerDialog
                colorDialog.StartingColor = color

                If colorDialog.ShowDialog() = True Then
                    TxtBorderColor.Background = New SolidColorBrush(colorDialog.SelectedColor)
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Handles the Click event of the LnkButtonBgColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub lnkButtonBgColor_OnClick(sender As Object, e As RoutedEventArgs)
            Dim backgroundBrush As SolidColorBrush = TryCast(TxtBgColor.Background, SolidColorBrush)
            Dim color As New System.Windows.Media.Color()
            If backgroundBrush IsNot Nothing Then
                color = backgroundBrush.Color
            Else
                Dim convertFromString As Object = System.Windows.Media.ColorConverter.ConvertFromString("Black")
                If convertFromString IsNot Nothing Then
                    color = DirectCast(convertFromString, System.Windows.Media.Color)
                End If
            End If

            Using colorDialog As IColorPickerDialog = _dialogService.ColorPickerDialog
                colorDialog.StartingColor = color

                If colorDialog.ShowDialog() = True Then
                    TxtBgColor.Background = New SolidColorBrush(colorDialog.SelectedColor)
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Handles the Checked and Unchecked events of the ChkBackgroundPicture control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        Private Sub chkBackgroundPicture_OnCheckedChanged(sender As Object, e As RoutedEventArgs)
            If Not IsInitialized Then
                Return
            End If

            LnkBtnBackgroundPicture.IsEnabled = ChkBackgroundPicture.IsChecked = True
        End Sub

        ''' <summary>
        ''' Handles the MouseLeftButtonDown event of the DialogHeader control.
        ''' </summary>
        ''' <param name="sender">The event source.</param>
        ''' <param name="e">The event data.</param>
        Private Sub DialogHeader_MouseLeftButtonDown(sender As Object, e As MouseButtonEventArgs)
            DragMove()
        End Sub

        ''' <summary>
        ''' Handles the OnRightButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        Private Sub TwoButtonPanel_OnRightButtonClicked(sender As Object, e As RoutedEventArgs)
             'cancel button

            Close()
        End Sub

        ''' <summary>
        ''' Handles the OnLeftButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub TwoButtonPanel_OnLeftButtonClicked(sender As Object, e As RoutedEventArgs)
           'ok button

            DialogResult = True
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

        ''' <summary>
        ''' Gets and sets owner window for this window
        ''' </summary>
        Public Overloads Property Owner() As Window Implements IDialog.Owner
            Get
                Return MyBase.Owner
            End Get
            Set(value As Window)
                MyBase.Owner = value
            End Set
        End Property

    End Class
End Namespace