﻿
Imports System.Windows
Imports System.Windows.Input
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Dialogs
Imports SpiceLogic.WpfHtmlEditor.Domain.Services

Namespace Dialogs

    ''' <summary>
    ''' Class YouTubeVideoInsertDialog
    ''' </summary>
    Partial Public Class YouTubeVideoInsertDialog
        Implements IYouTubeVideoInsertDialog

        ''' <summary>
        ''' The _the original element
        ''' </summary>
        Private _theOriginalElement As YouTubeVideoElement

        ''' <summary>
        ''' Initializes a new instance of the <see cref="YouTubeVideoInsertDialog" /> class.
        ''' </summary>
        Public Sub New()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As YouTubeVideoElement Implements SpiceLogic.HtmlEditorControl.Domain.Dialogs.IYouTubeVideoInsertDialogBase.Element
            Get
                If _theOriginalElement Is Nothing Then
                    Dim theElement As New YouTubeVideoElement() With { _
                        .URL = TxtUrl.Text.Trim(), _
                        .Width = TxtWidth.Text, _
                        .Height = TxtHeight.Text _
                    }

                    Return theElement
                End If

                _theOriginalElement.URL = TxtUrl.Text.Trim()
                _theOriginalElement.Width = TxtWidth.Text
                _theOriginalElement.Height = TxtHeight.Text
                _theOriginalElement.CSSStyle = TxtCssStyle.Text

                Return _theOriginalElement
            End Get
            Set(value As YouTubeVideoElement)
                _theOriginalElement = value
                TxtUrl.Text = value.URL
                TxtHeight.Text = value.Height
                TxtWidth.Text = value.Width
                TxtCssStyle.Text = value.CSSStyle
            End Set
        End Property


        Public Sub Dispose() Implements IDisposable.Dispose
        End Sub


        ''' <summary>
        ''' Handles the OnRightButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub TwoButtonPanel_OnRightButtonClicked(sender As Object, e As RoutedEventArgs)
            'cancel button

            Close()          
        End Sub

        ''' <summary>
        ''' Handles the OnLeftButtonClicked event of the TwoButtonPanel control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub TwoButtonPanel_OnLeftButtonClicked(sender As Object, e As RoutedEventArgs)
            'OK button

            If String.IsNullOrEmpty(TxtUrl.Text) Then
                MessageBox.Show("The YouTube URL cannot be empty.")
                TxtUrl.Focus()
                Return
            End If

            Dim theUrl As String = TxtUrl.Text.Trim()
            If theUrl = String.Empty Then
                MessageBox.Show("The YouTube URL cannot be empty.")
                TxtUrl.Focus()
                Return
            End If

            If Not theUrl.ToLower().Contains("youtube.com") Then
                MessageBox.Show("The URL you provided does not contain the YouTube Domain name", "Invalid URL")
                Return
            End If

            Me.DialogResult = True
        End Sub

        ''' <summary>
        ''' Handles the MouseLeftButtonDown event of the DialogHeader control
        ''' </summary>
        ''' <param name="sender">The event source</param>
        ''' <param name="e">The event data</param>
        Private Sub DialogHeader_MouseLeftButtonDown(sender As Object, e As MouseButtonEventArgs)
            DragMove()
        End Sub


        ''' <summary>
        ''' Gets and sets owner window for this window
        ''' </summary>
        Public Overloads Property Owner() As Window Implements IDialog.Owner
            Get
                Return MyBase.Owner
            End Get
            Set(value As Window)
                MyBase.Owner = value
            End Set
        End Property

        ''' <summary>
        ''' Shows this window as a dialog
        ''' </summary>
        ''' <returns>Returns <c>true</c> if closed using button OK</returns>
        Public Overloads Function ShowDialog() As Boolean? Implements IDialogBase.ShowDialog
            Return MyBase.ShowDialog()
        End Function

    End Class
End Namespace